@@ -154,10 +154,10 @@ resfiles = [
 	"ref/ClayRGB1998.gam",
 	"ref/sRGB.gam",
 	"ref/verify_extended.ti1",
-	"ti1/d3-e4-s2-g25-m0-b0-f0.ti1",
-	"ti1/d3-e4-s3-g49-m3-b0-f0.ti1",
-	"ti1/d3-e4-s4-g49-m4-b0-f0.ti1",
-	"ti1/d3-e4-s5-g49-m5-b0-f0.ti1",
+	"ti1/d3-e4-s2-g28-m0-b0-f0.ti1",
+	"ti1/d3-e4-s3-g52-m3-b0-f0.ti1",
+	"ti1/d3-e4-s4-g52-m4-b0-f0.ti1",
+	"ti1/d3-e4-s5-g52-m5-b0-f0.ti1",
 	"xrc/extra.xrc",
 	"xrc/gamap.xrc",
 	"xrc/main.xrc",
@@ -225,7 +225,7 @@ def check_3dlut_format(devicename):
 					getcfg("3dlut.bitdepth.output") == 12)
 
 
-def getbitmap(name, display_missing_icon=True, scale=True):
+def getbitmap(name, display_missing_icon=True, scale=True, use_mask=False):
 	"""
 	Create (if necessary) and return a named bitmap.
 	
@@ -262,17 +262,24 @@ def getbitmap(name, display_missing_icon=True, scale=True):
 			w = int(round(w * scale))
 			h = int(round(h * scale))
 		if parts[-1] == "empty":
-			bitmaps[name] = wx.EmptyBitmap(w, h, depth=-1)
-			dc = wx.MemoryDC()
-			dc.SelectObject(bitmaps[name])
-			dc.SetBackground(wx.Brush("black"))
-			dc.Clear()
-			dc.SelectObject(wx.NullBitmap)
-			bitmaps[name].SetMaskColour("black")
+			if use_mask and sys.platform == "win32":
+				bmp = wx.EmptyBitmap(w, h)
+				bmp.SetMaskColour(wx.Colour(0, 0, 0))
+			else:
+				bmp = wx.EmptyBitmapRGBA(w, h, 255, 0, 255, 0)
 		else:
 			if parts[-1].startswith(appname):
 				parts[-1] = parts[-1].lower()
 			oname = parts[-1]
+			if "#" in oname:
+				# Hex format, RRGGBB or RRGGBBAA
+				oname, color = oname.split("#", 1)
+				parts[-1] = oname
+			else:
+				color = None
+			inverted = oname.endswith("-inverted")
+			if inverted:
+				oname = parts[-1] = oname.split("-inverted")[0]
 			name2x = oname + "@2x"
 			name4x = oname + "@4x"
 			path = None
@@ -324,23 +331,30 @@ def getbitmap(name, display_missing_icon=True, scale=True):
 				if path or scale == 1:
 					break
 			if path:
-				bitmaps[name] = wx.Bitmap(path)
+				bmp = wx.Bitmap(path)
+				if not bmp.IsOk():
+					path = None
+			if path:
+				img = None
 				if scale > 1 and i:
 					rescale = False
 					if i in (1, 2):
 						# HighDPI support. 4x/2x version, determine scaled size
-						w, h = [int(round(v / (2 * (3 - i)) * scale)) for v in bitmaps[name].Size]
+						w, h = [int(round(v / (2 * (3 - i)) * scale)) for v in bmp.Size]
 						rescale = True
 					elif len(size) == 2:
 						# HighDPI support. Icon
 						rescale = True
-					if rescale and (bitmaps[name].Size[0] != w or
-									bitmaps[name].Size[1] != h):
+					if rescale and (bmp.Size[0] != w or
+									bmp.Size[1] != h):
 						# HighDPI support. Rescale
-						img = bitmaps[name].ConvertToImage()
-						if not hasattr(wx, "IMAGE_QUALITY_BILINEAR"):
+						img = bmp.ConvertToImage()
+						if (not hasattr(wx, "IMAGE_QUALITY_BILINEAR") or
+							oname == "list-add"):
+							# In case bilinear is not supported, and to prevent
+							# black borders after resizing for some images
 							quality = wx.IMAGE_QUALITY_NORMAL
-						elif oname == "rgbsquares":
+						elif oname in ():
 							# Hmm. Everything else looks great with bicubic,
 							# but this one gets jaggy unless we use bilinear
 							quality = wx.IMAGE_QUALITY_BILINEAR
@@ -349,20 +363,69 @@ def getbitmap(name, display_missing_icon=True, scale=True):
 						else:
 							quality = wx.IMAGE_QUALITY_BILINEAR
 						img.Rescale(w, h, quality=quality)
-						bitmaps[name] = img.ConvertToBitmap()
-			else:
+				factors = None
+				if (not inverted and len(parts) > 2 and parts[-3] == "icons" and
+					max(wx.SystemSettings.GetColour(wx.SYS_COLOUR_WINDOW)[:3]) < 102):
+					if not img:
+						img = bmp.ConvertToImage()
+					if img.IsBW():
+						inverted = True
+				# Invert after resize (avoids jaggies)
+				if inverted:
+					if not img:
+						img = bmp.ConvertToImage()
+					img.Invert()
+					alpha = wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNTEXT).alpha
+					if oname in ["applications-system", "color",
+								 "document-open", "document-save-as",
+								 "edit-delete", "image-x-generic", "info",
+								 "install", "list-add", "package-x-generic",
+								 "question", "rgbsquares",
+								 "stock_3d-color-picker", "stock_lock",
+								 "stock_lock-open", "stock_refresh", "web",
+								 "window-center", "zoom-best-fit", "zoom-in",
+								 "zoom-original", "zoom-out"]:
+						# Scale 85 to 255 and adjust alpha
+						factors = (3, 3, 3, alpha / 255.0)
+					else:
+						# Only adjust alpha
+						factors = (1, 1, 1, alpha / 255.0)
+				if color or factors:
+					if not img:
+						img = bmp.ConvertToImage()
+					if color:
+						# Hex format, RRGGBB or RRGGBBAA
+						R = int(color[0:2], 16) / 255.0
+						G = int(color[2:4], 16) / 255.0
+						B = int(color[4:6], 16) / 255.0
+						if len(color) > 6:
+							alpha = int(color[6:8], 16) / 255.0
+						else:
+							alpha = 1.0
+					else:
+						R, G, B = factors[:3]
+						if len(factors) > 3:
+							alpha = factors[3]
+						else:
+							alpha = 1.0
+					img = img.AdjustChannels(R, G, B, alpha)
+				if img:
+					bmp = img.ConvertToBitmap()
+					if not bmp.IsOk():
+						path = None
+			if not path:
 				img = wx.EmptyImage(w, h)
 				img.SetMaskColour(0, 0, 0)
 				img.InitAlpha()
 				bmp = img.ConvertToBitmap()
-				bitmaps[name] = bmp
 				dc = wx.MemoryDC()
-				dc.SelectObject(bitmaps[name])
+				dc.SelectObject(bmp)
 				if display_missing_icon:
-					bmp = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE,
+					art = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE,
 												   size=(w, h))
-					dc.DrawBitmap(bmp, 0, 0, True)
+					dc.DrawBitmap(art, 0, 0, True)
 				dc.SelectObject(wx.NullBitmap)
+		bitmaps[name] = bmp
 	return bitmaps[name]
 
 
@@ -440,9 +503,11 @@ def get_display_number(display_no):
 	except IndexError:
 		return 0
 	else:
+		if display.endswith(" [PRIMARY]"):
+			display = " ".join(display.split(" ")[:-1])
 		for i in xrange(wx.Display.GetCount()):
 			geometry = "%i, %i, %ix%i" % tuple(wx.Display(i).Geometry)
-			if display.find("@ " + geometry) > -1:
+			if display.endswith("@ " + geometry):
 				if debug:
 					from log import safe_print
 					safe_print("[D] Found display %s at index %i" % 
@@ -468,10 +533,13 @@ def get_icon_bundle(sizes, name):
 	iconbundle = wx.IconBundle()
 	if not sizes:
 		# Assume ICO format
-		ico = get_data_path("theme/icons/%s.ico" % name)
-		if ico:
-			iconbundle.AddIconFromFile(ico, wx.BITMAP_TYPE_ICO)
-			return iconbundle
+		pth = get_data_path("theme/icons/%s.ico" % name)
+		if pth:
+			ico = wx.Icon(pth)
+			if ico.IsOk():
+				iconbundle.AddIcon(ico)
+				return iconbundle
+		sizes = [16]
 	for size in sizes:
 		iconbundle.AddIcon(get_bitmap_as_icon(size, name, False))
 	return iconbundle
@@ -498,10 +566,11 @@ def get_measureframe_dimensions(dimensions_measureframe=None, percent=10):
 	return ",".join([str(min(n, 50)) for n in dimensions_measureframe])
 
 
-def geticon(size, name, scale=True):
+def geticon(size, name, scale=True, use_mask=False):
 	""" Convenience function for getbitmap('theme/icons/<size>/<name>'). """
 	return getbitmap("theme/icons/%(size)sx%(size)s/%(name)s" % 
-					 {"size": size, "name": name}, scale=scale)
+					 {"size": size, "name": name}, scale=scale,
+					 use_mask=use_mask)
 
 
 def get_data_path(relpath, rex=None):
@@ -633,7 +702,9 @@ cfg = ConfigParser.RawConfigParser()
 cfg.optionxform = str
 
 valid_ranges = {
-	"3dlut.hdr_peak_luminance": [120, 10000],
+	"3dlut.hdr_peak_luminance": [120.0, 10000.0],
+	"3dlut.hdr_minmll": [0.0, 0.1],
+	"3dlut.hdr_maxmll": [120.0, 10000.0],
 	"3dlut.trc_gamma": [0.000001, 10],
 	"3dlut.trc_output_offset": [0.0, 1.0],
 	"app.port": [1, 65535],
@@ -682,7 +753,7 @@ valid_values = {
 	"3dlut.image.order": ["rgb", "bgr"],
 	"3dlut.rendering_intent": intents,
 	"3dlut.size": [5, 9, 16, 17, 24, 32, 33, 64, 65],
-	"3dlut.trc": ["bt1886", "customgamma", "gamma2.2", "smpte2084.hardclip", "smpte2084.rolloffclip"],
+	"3dlut.trc": ["bt1886", "customgamma", "gamma2.2", "smpte2084.hardclip", "smpte2084.rolloffclip", "hlg"],
 	"3dlut.trc_gamma_type": ["b", "B"],
 	"calibration.quality": ["v", "l", "m", "h", "u"],
 	"colorimeter_correction.observer": observers,
@@ -715,6 +786,7 @@ valid_values = {
 	"profile.quality.b2a": ["l", "m", "h", "u", "n", None],
 	"profile.b2a.hires.size": [-1, 9, 17, 33, 45, 65],
 	"profile.type": ["g", "G", "l", "s", "S", "x", "X"],
+	"profile_loader.tray_icon_animation_quality": [0, 1, 2],
 	"synthprofile.black_point_compensation": [0, 1],
 	"synthprofile.trc_gamma_type": ["g", "G"],
 	"tc_algo": ["", "t", "r", "R", "q", "Q", "i", "I"],  # Q = Argyll >= 1.1.0
@@ -768,8 +840,10 @@ defaults = {
 	"3dlut.format": "cube",
 	"3dlut.gamap.use_b2a": 0,
 	"3dlut.hdr_display": 0,
-	"3dlut.hdr_maxcll": 10000,
-	"3dlut.hdr_peak_luminance": 400,
+	"3dlut.hdr_minmll": 0.0,
+	"3dlut.hdr_maxmll": 10000.0,
+	"3dlut.hdr_peak_luminance": 400.0,
+	"3dlut.hdr_ambient_luminance": 5.0,
 	"3dlut.image.layout": "h",
 	"3dlut.image.order": "rgb",
 	"3dlut.input.profile": "",
@@ -777,6 +851,7 @@ defaults = {
 	"3dlut.enable": 1,
 	"3dlut.output.profile": "",
 	"3dlut.output.profile.apply_cal": 1,
+	"3dlut.preserve_sync": 0,
 	"3dlut.rendering_intent": "aw",
 	"3dlut.use_abstract_profile": 0,
 	"3dlut.size": 65,
@@ -858,7 +933,7 @@ defaults = {
 	"gamap_saturation_intent": "s",
 	"gamap_default_intent": "p",
 	"gamma": 2.2,
-	"iccgamut.surface_detail": 6.0,
+	"iccgamut.surface_detail": 10.0,
 	"instruments": "",
 	"last_launch": "99",
 	"log.autoshow": 0,
@@ -936,7 +1011,9 @@ defaults = {
 	"position.synthiccframe.y": 50,
 	"position.tcgen.x": 50,
 	"position.tcgen.y": 50,
-	"profile.black_point_compensation": 0,
+	# Force black point compensation due to OS X
+	# bugs with non BPC profiles
+	"profile.black_point_compensation": 0 if sys.platform != "darwin" else 1,
 	"profile.black_point_correction": 0.0,
 	"profile.create_gamut_views": 1,
 	"profile.install_scope": "l" if (sys.platform != "win32" and 
@@ -966,13 +1043,17 @@ defaults = {
 	"profile.b2a.hires.size": -1,
 	"profile.b2a.hires.smooth": 1,
 	"profile.save_path": storage, # directory
-	"profile.type": "X",
+	# Force profile type to single shaper + matrix
+	# due to OS X bugs with cLUT profiles and
+	# matrix profiles with individual shaper curves
+	"profile.type": "X" if sys.platform != "darwin" else "S",
 	"profile.update": 0,
 	"profile_loader.buggy_video_drivers":  ";".join(["*"]),
 	"profile_loader.check_gamma_ramps": 1,
 	"profile_loader.error.show_msg": 1,
 	"profile_loader.exceptions": "",
 	"profile_loader.fix_profile_associations": 1,
+	"profile_loader.ignore_unchanged_gamma_ramps": 1,
 	"profile_loader.known_apps": ";".join(["basiccolor display.exe",
 										   "calclient.exe",
 										   "coloreyes display pro.exe",
@@ -1003,7 +1084,9 @@ defaults = {
 	"profile_loader.known_window_classes": ";".join(["CalClient.exe"]),
 	"profile_loader.quantize_bits": 16,
 	"profile_loader.reset_gamma_ramps": 0,
+	"profile_loader.show_notifications": 0,
 	"profile_loader.track_other_processes": 1,
+	"profile_loader.tray_icon_animation_quality": 2,
 	"profile_loader.use_madhcnet": 0,
 	"profile_loader.verify_calibration": 0,
 	"recent_cals": "",
@@ -1072,7 +1155,10 @@ defaults = {
 	"tc_vrml_use_D50": 0,
 	"tc_white_patches": 4,
 	"tc.show": 0,
-	"testchart.auto_optimize": 4,
+	# Profile type forced to matrix due to
+	# OS X bugs with cLUT profiles. Set
+	# smallest testchart.
+	"testchart.auto_optimize": 4 if sys.platform != "darwin" else 1,
 	"testchart.file": "auto",
 	"testchart.file.backup": "auto",
 	"testchart.patch_sequence": "optimize_display_response_delay",
@@ -1304,7 +1390,7 @@ def get_display_profile(display_no=None):
 	except Exception, exception:
 		from log import _safe_print, log
 		_safe_print("ICCP.get_display_profile(%s):" % display_no, 
-					exception, fn=log)
+					safe_unicode(exception), fn=log)
 
 
 standard_profiles = []
@@ -1702,3 +1788,10 @@ def writecfg(which="user", worker=None, module=None, options=()):
 
 _init_testcharts()
 runtype = runtimeconfig(pyfile)
+
+if isapp and not os.getenv("SSL_CERT_FILE"):
+	# Use our bundled CA file
+	# https://github.com/certifi/python-certifi/blob/master/certifi/cacert.pem
+	cafile = get_data_path("cacert.pem")
+	if cafile:
+		os.environ["SSL_CERT_FILE"] = cafile.encode(fs_enc, "replace")
