@@ -80,6 +80,7 @@ config = {"data": ["tests/*.icc"],
 				  "theme/*.png",
 				  "theme/*.css",
 				  "theme/*.js",
+				  "theme/*.svg",
 				  "theme/icons/favicon.ico",
 				  "theme/slimbox2/*.css",
 				  "theme/slimbox2/*.js"],
@@ -121,7 +122,11 @@ config = {"data": ["tests/*.icc"],
 								  "theme/icons/10x10/*.png",
 								  "theme/icons/16x16/*.png",
 								  "theme/icons/32x32/*.png",
+								  "theme/icons/48x48/*.png",
 								  "theme/icons/72x72/*.png",
+								  "theme/icons/128x128/*.png",
+								  "theme/icons/256x256/*.png",
+								  "theme/icons/512x512/*.png",
 								  "theme/jet_anim/*.png",
 								  "theme/patch_anim/*.png",
 								  "theme/splash_anim/*.png",
@@ -171,7 +176,7 @@ plist_dict = {"CFBundleDevelopmentRegion": "English",
 			  "CFBundleSignature": "????",
 			  "CFBundleVersion": ".".join(map(str, version_tuple)),
 			  "NSHumanReadableCopyright": u"© %s %s" % (strftime("%Y"), author),
-			  "LSMinimumSystemVersion": "10.5.0"}
+			  "LSMinimumSystemVersion": "10.6.0"}
 
 
 class Target:
@@ -358,6 +363,8 @@ def setup():
 
 	sys.path.insert(1, os.path.join(os.path.dirname(pydir), "util"))
 
+	current_findall = distutils.filelist.findall
+
 	if use_setuptools:
 		if "--use-setuptools" in sys.argv[1:] and not \
 		   os.path.exists("use-setuptools"):
@@ -371,12 +378,28 @@ def setup():
 			from setuptools import setup, Extension
 			setuptools = True
 			print "using setuptools"
+			current_findall = distutils.filelist.findall
 		except ImportError:
 			pass
 	else:
 		if os.path.exists("use-setuptools"):
 			os.remove("use-setuptools")
 
+	if distutils.filelist.findall is current_findall:
+		# Fix traversing unneeded dirs which can take a long time (minutes)
+		def findall(dir=os.curdir, original=distutils.filelist.findall, listdir=os.listdir,
+					basename=os.path.basename):
+			os.listdir = lambda path: filter(lambda entry: entry not in ("build",
+																		 "dist") and
+														   not entry.startswith("."),
+											 listdir(path))
+			try:
+				return original(dir)
+			finally:
+				os.listdir = listdir
+
+		distutils.filelist.findall = findall
+
 	if not setuptools:
 		from distutils.core import setup, Extension
 		print "using distutils"
@@ -497,6 +520,10 @@ def setup():
 			if is_rpm_build:
 				doc = os.path.join(os.path.sep, "usr", doc)
 
+	# Use bundled CA file
+	if sdist or do_py2app:
+		config["package_data"][name].append("cacert.pem")
+
 	# on Mac OS X and Windows, we want data files in the package dir
 	# (package_data will be ignored when using py2exe)
 	package_data = {
@@ -506,10 +533,7 @@ def setup():
 	}
 	if sdist and sys.platform in ("darwin", "win32"):
 		package_data[name].extend(["theme/icons/22x22/*.png",
-								   "theme/icons/24x24/*.png",
-								   "theme/icons/48x48/*.png",
-								   "theme/icons/128x128/*.png",
-								   "theme/icons/256x256/*.png"])
+								   "theme/icons/24x24/*.png"])
 	if sys.platform == "win32" and not do_py2exe:
 		package_data[name].append("theme/icons/*.ico")
 	# Scripts
@@ -612,22 +636,35 @@ def setup():
 							if os.path.isfile(fname):
 								data_files.extend([(entry[0], [fname])])
 		for dname in ("10x10", "16x16", "22x22", "24x24", "32x32", "48x48",
-					  "72x72", "128x128", "256x256"):
-			# Only the 10x10, 16x16, 32x32 and 72x72 icons are used exclusively
-			# by the app, the other sizes of the app icon are used for the 
-			# desktop entry under Linux
+					  "72x72", "128x128", "256x256", "512x512"):
+			# Get all the icons needed, depending on platform
+			# Only the icon sizes 10, 16, 32, 72, 256 and 512 include icons
+			# that are used exclusively for UI elements.
+			# These should be installed in an app-specific location, e.g.
+			# under Linux $XDG_DATA_DIRS/DisplayCAL/theme/icons/
+			# The app icon sizes 16, 32, 48 and 256 (128 under Mac OS X),
+			# which are used for taskbar icons and the like, as well as the
+			# other sizes can be installed in a generic location, e.g.
+			# under Linux $XDG_DATA_DIRS/icons/hicolor/<size>/apps/
+			# Generally, icon filenames starting with the lowercase app name
+			# should be installed in the generic location.
 			icons = []
 			desktopicons = []
 			if sys.platform == "darwin":
-				largest_icon_size = "128x128"
+				largest_iconbundle_icon_size = "128x128"
 			else:
-				largest_icon_size = "256x256"
+				largest_iconbundle_icon_size = "256x256"
 			for iconpath in glob.glob(os.path.join(pydir, "theme", "icons", 
 												   dname, "*.png")):
 				if not os.path.basename(iconpath).startswith(name.lower()) or (
 					sys.platform in ("darwin", "win32") and 
-					dname in ("16x16", "32x32", "48x48", largest_icon_size)):
-					# We need all the sizes we use in get_icon_bundle
+					dname in ("16x16", "32x32", "48x48",
+							  largest_iconbundle_icon_size)):
+					# In addition to UI element icons, we also need all the app
+					# icons we use in get_icon_bundle under macOS/Windows,
+					# otherwise they wouldn't be included (under Linux, these
+					# are included for installation to the system-wide icon
+					# theme location instead)
 					icons.append(iconpath)
 				elif sys.platform not in ("darwin", "win32"):
 					desktopicons.append(iconpath)
@@ -670,7 +707,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					lines.append(o)
 				print o.rstrip()
 			if len(lines):
-				os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.3'
+				os.environ['MACOSX_DEPLOYMENT_TARGET'] = '10.5'
 				sp.call(lines[-1], shell = True)  # fix the library
 	else:
 		macros = [("UNIX", None)]
@@ -715,7 +752,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 
 	attrs = {
 		"author": author_ascii,
-		"author_email": author_email,
+		"author_email": author_email.replace("@", "_at_"),
 		"classifiers": [
 			"Development Status :: 5 - Production/Stable",
 			"Environment :: MacOS X",
@@ -1040,6 +1077,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					"README-fr.html",
 					"argyll_instruments.json",
 					"beep.wav",
+					"cacert.pem",
 					"camera_shutter.wav",
 					"ColorLookupTable.fx",
 					name.lower() + ".desktop",
@@ -1172,7 +1210,6 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 		manifest_in.append("recursive-include %s %s %s" % 
 						   (os.path.join(name, "theme", "icons"), 
 						   "*.icns", "*.ico"))
-		manifest_in.append("recursive-include %s %s" % ("autopackage", "*"))
 		manifest_in.append("include " + os.path.join("man", "*.1"))
 		manifest_in.append("recursive-include %s %s" % ("misc", "*"))
 		if skip_instrument_conf_files:
