@@ -1,5 +1,8 @@
 # -*- coding: utf-8 -*-
 
+import fnmatch
+import ctypes
+import errno
 import glob
 import locale
 import os
@@ -9,10 +12,35 @@ import subprocess as sp
 import sys
 import tempfile
 import time
-from os.path import join
+
+if sys.platform not in ("darwin", "win32"):
+	# Linux
+	import grp
+	import pwd
+
+try:
+	reloaded
+except NameError:
+	# First import. All fine
+	reloaded = 0
+else:
+	# Module is being reloaded. NOT recommended.
+	reloaded += 1
+	import warnings
+	warnings.warn("Module %s is being reloaded. This is NOT recommended." %
+				  __name__, RuntimeWarning)
+	warnings.warn("Implicitly reloading builtins", RuntimeWarning)
+	if sys.platform == "win32":
+		reload(__builtin__)
+	warnings.warn("Implicitly reloading os", RuntimeWarning)
+	reload(os)
+	warnings.warn("Implicitly reloading os.path", RuntimeWarning)
+	reload(os.path)
+	if sys.platform == "win32":
+		warnings.warn("Implicitly reloading win32api", RuntimeWarning)
+		reload(win32api)
 
 if sys.platform == "win32":
-	import ctypes
 	from win32file import *
 	from winioctlcon import FSCTL_GET_REPARSE_POINT
 
@@ -26,7 +54,7 @@ from encoding import get_encodings
 
 fs_enc = get_encodings()[1]
 
-os._listdir = os.listdir
+_listdir = os.listdir
 
 if sys.platform == "win32":
 	# Add support for long paths (> 260 chars)
@@ -35,7 +63,7 @@ if sys.platform == "win32":
 	import winerror
 	import win32api
 
-	__builtin__._open = __builtin__.open
+	_open = __builtin__.open
 
 
 	def retry_sharing_violation_factory(fn, delay=0.25, maxretries=20):
@@ -57,115 +85,116 @@ if sys.platform == "win32":
 
 
 	def open(path, *args, **kwargs):
-		return __builtin__._open(make_win32_compatible_long_path(path), *args,
+		""" Wrapper around __builtin__.open dealing with win32 long paths """
+		return _open(make_win32_compatible_long_path(path), *args,
 								 **kwargs)
 
 	__builtin__.open = open
 
 
-	os._access = os.access
+	_access = os.access
 
 	def access(path, mode):
-		return os._access(make_win32_compatible_long_path(path), mode)
+		return _access(make_win32_compatible_long_path(path), mode)
 
 	os.access = access
 
 
-	os.path._exists = os.path.exists
+	_exists = os.path.exists
 
 	def exists(path):
-		return os.path._exists(make_win32_compatible_long_path(path))
+		return _exists(make_win32_compatible_long_path(path))
 
 	os.path.exists = exists
 
 
-	os.path._isdir = os.path.isdir
+	_isdir = os.path.isdir
 
 	def isdir(path):
-		return os.path._isdir(make_win32_compatible_long_path(path))
+		return _isdir(make_win32_compatible_long_path(path))
 
 	os.path.isdir = isdir
 
 
-	os.path._isfile = os.path.isfile
+	_isfile = os.path.isfile
 
 	def isfile(path):
-		return os.path._isfile(make_win32_compatible_long_path(path))
+		return _isfile(make_win32_compatible_long_path(path))
 
 	os.path.isfile = isfile
 
 
 	def listdir(path):
-		return os._listdir(make_win32_compatible_long_path(path))
+		return _listdir(make_win32_compatible_long_path(path))
 
 
-	os._lstat = os.lstat
+	_lstat = os.lstat
 
 	def lstat(path):
-		return os._lstat(make_win32_compatible_long_path(path))
+		return _lstat(make_win32_compatible_long_path(path))
 
 	os.lstat = lstat
 
 
-	os._mkdir = os.mkdir
+	_mkdir = os.mkdir
 
 	def mkdir(path, mode=0777):
-		return os._mkdir(make_win32_compatible_long_path(path, 247), mode)
+		return _mkdir(make_win32_compatible_long_path(path, 247), mode)
 
 	os.mkdir = mkdir
 
 
-	os._makedirs = os.makedirs
+	_makedirs = os.makedirs
 
 	def makedirs(path, mode=0777):
-		return os._makedirs(make_win32_compatible_long_path(path, 247), mode)
+		return _makedirs(make_win32_compatible_long_path(path, 247), mode)
 
 	os.makedirs = makedirs
 
 
-	os._remove = os.remove
+	_remove = os.remove
 
 	def remove(path):
-		return os._remove(make_win32_compatible_long_path(path))
+		return _remove(make_win32_compatible_long_path(path))
 
 	os.remove = retry_sharing_violation_factory(remove)
 
 
-	os._rename = os.rename
+	_rename = os.rename
 
 	def rename(src, dst):
 		src, dst = [make_win32_compatible_long_path(path) for path in
 					(src, dst)]
-		return os._rename(src, dst)
+		return _rename(src, dst)
 
 	os.rename = retry_sharing_violation_factory(rename)
 
 
-	os._stat = os.stat
+	_stat = os.stat
 
 	def stat(path):
-		return os._stat(make_win32_compatible_long_path(path))
+		return _stat(make_win32_compatible_long_path(path))
 
 	os.stat = stat
 
 
-	os._unlink = os.unlink
+	_unlink = os.unlink
 
 	def unlink(path):
-		return os._unlink(make_win32_compatible_long_path(path))
+		return _unlink(make_win32_compatible_long_path(path))
 
 	os.unlink = retry_sharing_violation_factory(unlink)
 
 
-	win32api._GetShortPathName = win32api.GetShortPathName
+	_GetShortPathName = win32api.GetShortPathName
 
 	def GetShortPathName(path):
-		return win32api._GetShortPathName(make_win32_compatible_long_path(path))
+		return _GetShortPathName(make_win32_compatible_long_path(path))
 
 	win32api.GetShortPathName = GetShortPathName
 else:
 	def listdir(path):
-		paths = os._listdir(path)
+		paths = _listdir(path)
 		if isinstance(path, unicode):
 			# Undecodable filenames will still be string objects. Ignore them.
 			paths = filter(lambda path: isinstance(path, unicode), paths)
@@ -185,6 +214,55 @@ def quote_args(args):
 	return args_out
 
 
+def dlopen(name):
+	try:
+		return ctypes.CDLL(name)
+	except:
+		pass
+
+
+def find_library(pattern, arch=None):
+	"""
+	Use ldconfig cache to find installed library.
+	
+	Can use fnmatch-style pattern matching.
+	
+	"""
+	try:
+		p = sp.Popen(["/sbin/ldconfig", "-p"], stdout=sp.PIPE)
+		stdout, stderr = p.communicate()
+	except:
+		return
+	if not arch:
+		try:
+			p = sp.Popen(["file", "-L", sys.executable], stdout=sp.PIPE)
+			file_stdout, file_stderr = p.communicate()
+		except:
+			pass
+		else:
+			# /usr/bin/python2.7: ELF 64-bit LSB shared object, x86-64,
+			# version 1 (SYSV), dynamically linked, interpreter
+			# /lib64/ld-linux-x86-64.so.2, for GNU/Linux 3.2.0,
+			# BuildID[sha1]=41a1f0d4da3afee8f22d1947cc13a9f33f59f2b8, stripped
+			parts = file_stdout.split(",")
+			if len(parts) > 1:
+				arch = parts[1].strip()
+	for line in stdout.splitlines():
+		# libxyz.so (libc6,x86_64) => /lib64/libxyz.so.1
+		parts = line.split("=>", 1)
+		candidate = parts[0].split(None, 1)
+		if len(parts) < 2 or len(candidate) < 2:
+			continue
+		info = candidate[1].strip("( )").split(",")
+		if arch and len(info) > 1 and info[1].strip() != arch:
+			# Skip libs for wrong arch
+			continue
+		filename = candidate[0]
+		if fnmatch.fnmatch(filename, pattern):
+			path = parts[1].strip()
+			return path
+
+
 def expanduseru(path):
 	""" Unicode version of os.path.expanduser """
 	if sys.platform == "win32":
@@ -204,10 +282,10 @@ def expanduseru(path):
 			return path
 		else:
 			drive = getenvu('HOMEDRIVE', '')
-			userhome = join(drive, getenvu('HOMEPATH'))
+			userhome = os.path.join(drive, getenvu('HOMEPATH'))
 
 		if i != 1: #~user
-			userhome = join(dirname(userhome), path[1:i])
+			userhome = os.path.join(dirname(userhome), path[1:i])
 
 		return userhome + path[i:]
 	return unicode(os.path.expanduser(path), fs_enc)
@@ -292,6 +370,11 @@ def expandvarsu(path):
 	return unicode(os.path.expandvars(path), fs_enc)
 
 
+def fname_ext(path):
+	""" Get filename and extension """
+	return os.path.splitext(os.path.basename(path))
+
+
 def get_program_file(name, foldername):
 	""" Get path to program file """
 	if sys.platform == "win32":
@@ -323,6 +406,23 @@ def getenvu(name, default = None):
 		return var if isinstance(var, unicode) else unicode(var, fs_enc)
 
 
+def getgroups(username=None, names_only=False):
+	"""
+	Return a list of groups that user is member of, or groups of current
+	process if username not given
+	
+	"""
+	if username is None:
+		groups = [grp.getgrgid(g) for g in os.getgroups()]
+	else:
+		groups = [g for g in grp.getgrall() if username in g.gr_mem]
+		gid = pwd.getpwnam(username).pw_gid
+		groups.append(grp.getgrgid(gid))
+	if names_only:
+		groups = [g.gr_name for g in groups]
+	return groups
+
+
 def islink(path):
 	"""
 	Cross-platform islink implementation.
@@ -370,7 +470,7 @@ def launch_file(filepath):
 		kwargs["startupinfo"].wShowWindow = sp.SW_HIDE
 		kwargs["shell"] = True
 		kwargs["close_fds"] = True
-		retcode = sp.call('start "" /B "%s"' % filepath, **kwargs)
+		retcode = sp.call('start "" "%s"' % filepath, **kwargs)
 	elif which('xdg-open'):
 		retcode = sp.call(['xdg-open', filepath], **kwargs)
 	return retcode
@@ -392,6 +492,52 @@ def make_win32_compatible_long_path(path, maxpath=259):
 	return path
 
 
+def mkstemp_bypath(path, dir=None, text=False):
+	"""
+	Wrapper around mkstemp that uses filename and extension from path as prefix 
+	and suffix for the temporary file, and the directory component as temporary
+	file directory if 'dir' is not given.
+	
+	"""
+	fname, ext = fname_ext(path)
+	if not dir:
+		dir = os.path.dirname(path)
+	return tempfile.mkstemp(ext, fname + "-", dir, text)
+
+
+def mksfile(filename):
+	"""
+	Create a file safely and return (fd, abspath)
+	
+	If filename already exists, add '(n)' as suffix before extension (will
+	try up to os.TMP_MAX or 10000 for n)
+	
+	Basically, this works in a similar way as _mkstemp_inner from the
+	standard library 'tempfile' module.
+	
+	"""
+
+	flags = tempfile._bin_openflags
+
+	fname, ext = os.path.splitext(filename)
+
+	for seq in xrange(tempfile.TMP_MAX):
+		if not seq:
+			pth = filename
+		else:
+			pth = "%s(%i)%s" % (fname, seq, ext)
+		try:
+			fd = os.open(pth, flags, 0600)
+			tempfile._set_cloexec(fd)
+			return (fd, os.path.abspath(pth))
+		except OSError, e:
+			if e.errno == errno.EEXIST:
+				continue  # Try again
+			raise
+
+	raise IOError, (errno.EEXIST, "No usable temporary file name found")
+
+
 def movefile(src, dst, overwrite=True):
 	""" Move a file to another location.
 	
@@ -612,24 +758,47 @@ def which(executable, paths = None):
 	return None
 
 
-def whereis(filename):
-	for args in (["whereis", filename], ["locate", filename],
-				 ["ldconfig", "-p"]):
-		try:
-			p = sp.Popen(args, stdout=sp.PIPE)
-			stdout, stderr = p.communicate()
-		except:
-			pass
-		else:
-			stdout_lines = stdout.strip().split(os.linesep)
-			if args[0] == "ldconfig":
-				for line in stdout_lines:
-					if filename in line:
-						return line.split("=>").pop().strip()
-			else:
-				result = stdout_lines.pop().split(":", 1).pop().strip()
-				if result:
-					return result
+def whereis(names, bin=True, bin_paths=None, man=True, man_paths=None, src=True,
+			src_paths=None, unusual=False, list_paths=False):
+	"""
+	Wrapper around whereis
+	
+	"""
+	args = []
+	if bin:
+		args.append("-b")
+	if bin_paths:
+		args.append("-B")
+		args.extend(bin_paths)
+	if man:
+		args.append("-m")
+	if man_paths:
+		args.append("-M")
+		args.extend(man_paths)
+	if src:
+		args.append("-s")
+	if src_paths:
+		args.append("-S")
+		args.extend(src_paths)
+	if bin_paths or man_paths or src_paths:
+		args.append("-f")
+	if unusual:
+		args.append("-u")
+	if list_paths:
+		args.append("-l")
+	if isinstance(names, basestring):
+		names = [names]
+	p = sp.Popen(["whereis"] + args + names, stdout=sp.PIPE)
+	stdout, stderr = p.communicate()
+	result = {}
+	for line in stdout.strip().splitlines():
+		# $ whereis abc xyz
+		# abc: /bin/abc
+		# xyz: /bin/xyz /usr/bin/xyz
+		match = line.split(":", 1)
+		if match:
+			result[match[0]] = match[-1].split()
+	return result
 
 
 if sys.platform == "win32" and sys.getwindowsversion() >= (6, ):
