@@ -20,15 +20,16 @@ from lib.agw.fmresources import *
 from lib.agw.pygauge import PyGauge
 
 from config import (get_data_path, get_default_dpi, get_icon_bundle, getbitmap,
-					getcfg, setcfg)
+					getcfg, geticon, setcfg)
 from config import enc
 from log import get_file_logger
 from meta import name as appname
+from options import debug
 from ordereddict import OrderedDict
 from util_list import intlist
 from util_str import safe_unicode, wrap
-from wxwindows import (BaseApp, BaseFrame, FlatShadedButton,
-					   numpad_keycodes, wx_Panel)
+from wxwindows import (BaseApp, BaseFrame, FlatShadedButton, numpad_keycodes,
+					   nav_keycodes, processing_keycodes, wx_Panel)
 import audio
 import config
 import localization as lang
@@ -40,8 +41,10 @@ FGCOLOUR = wx.Colour(0x99, 0x99, 0x99)
 CRT = True
 
 def get_panel(parent, size=wx.DefaultSize):
+	scale = max(getcfg("app.dpi") / get_default_dpi(), 1.0)
+	size = tuple(int(round(v * scale)) for v in size)
 	panel = wx_Panel(parent, wx.ID_ANY, size=size)
-	if "--debug" in sys.argv[1:]:
+	if debug:
 		from random import randint
 		panel.SetBackgroundColour(wx.Colour(randint(0, 255), randint(0, 255), randint(0, 255)))
 		get_panel.i += 1
@@ -540,7 +543,9 @@ class DisplayAdjustmentPanel(wx_Panel):
 						   "G": (wx.Colour(0, 153, 0), wx.Colour(0, 255, 0)),
 						   "B": (wx.Colour(0, 0, 153), wx.Colour(0, 0, 255)),
 						   "L": (wx.Colour(153, 153, 153), wx.Colour(255, 255, 255))}
-		self.gauges[name] = PyGauge(self, size=(200, 8))
+		scale = max(getcfg("app.dpi") / get_default_dpi(), 1.0)
+		self.gauges[name] = PyGauge(self, size=(int(round(200 * scale)),
+												int(round(8 * scale))))
 		self.gauges[name].SetBackgroundColour(BORDERCOLOUR)
 		self.gauges[name].SetBarGradient(gaugecolors[name])
 		self.gauges[name].SetBorderColour(BORDERCOLOUR)
@@ -559,9 +564,11 @@ class DisplayAdjustmentPanel(wx_Panel):
 
 	def add_marker(self, direction="top"):
 		self.sizer.Add((1, 1))
+		scale = max(getcfg("app.dpi") / get_default_dpi(), 1.0)
 		self.sizer.Add(wx.StaticBitmap(self, -1,
 									   getbitmap("theme/marker_%s" % direction),
-									   size=(200, 10)))
+									   size=(int(round(200 * scale)),
+											 int(round(10 * scale)))))
 	
 	def add_txt(self, name, spacer=None, border=8):
 		checkmark = wx.StaticBitmap(self, wx.ID_ANY,
@@ -697,9 +704,12 @@ class DisplayAdjustmentFrame(BaseFrame):
 		self.indicator_panel.SetSizer(wx.BoxSizer(wx.HORIZONTAL))
 		self.indicator_panel.SetForegroundColour(FGCOLOUR)
 		self.btnsizer.Add(self.indicator_panel, flag=wx.EXPAND)
+		scale = max(getcfg("app.dpi") / get_default_dpi(), 1.0)
 		self.indicator_ctrl = wx.StaticBitmap(self.indicator_panel, wx.ID_ANY,
-											  getbitmap("10x10/empty"),
-											  size=(10, 10))
+											  geticon(10, "empty",
+													  use_mask=True),
+											  size=(int(round(10 * scale)),
+													int(round(10 * scale))))
 		self.indicator_ctrl.SetForegroundColour(FGCOLOUR)
 		self.indicator_panel.GetSizer().Add(self.indicator_ctrl, 
 											flag=wx.ALIGN_CENTER_VERTICAL)
@@ -730,11 +740,15 @@ class DisplayAdjustmentFrame(BaseFrame):
 		self.add_panel((12, 12), flag=wx.EXPAND)
 		
 		self.keyhandler = keyhandler
-		if sys.platform in ("darwin", "win32"):
-			# Use an accelerator table for space, 0-9, A-Z, numpad
-			keycodes = [ord(" ")] + range(ord("0"),
-										  ord("9")) + range(ord("A"),
-															ord("Z")) + numpad_keycodes
+		if sys.platform == "darwin":
+			# Use an accelerator table for tab, space, 0-9, A-Z, numpad,
+			# navigation keys and processing keys
+			keycodes = [wx.WXK_TAB, wx.WXK_SPACE]
+			keycodes.extend(range(ord("0"), ord("9")))
+			keycodes.extend(range(ord("A"), ord("Z")))
+			keycodes.extend(numpad_keycodes)
+			keycodes.extend(nav_keycodes)
+			keycodes.extend(processing_keycodes)
 			self.id_to_keycode = {}
 			for keycode in keycodes:
 				self.id_to_keycode[wx.NewId()] = keycode
@@ -742,6 +756,8 @@ class DisplayAdjustmentFrame(BaseFrame):
 			for id, keycode in self.id_to_keycode.iteritems():
 				self.Bind(wx.EVT_MENU, self.key_handler, id=id)
 				accels.append((wx.ACCEL_NORMAL, keycode, id))
+				if keycode == wx.WXK_TAB:
+					accels.append((wx.ACCEL_SHIFT, keycode, id))
 			self.SetAcceleratorTable(wx.AcceleratorTable(accels))
 		else:
 			self.Bind(wx.EVT_CHAR_HOOK, self.key_handler)
@@ -1023,16 +1039,18 @@ class DisplayAdjustmentFrame(BaseFrame):
 			keycode = event.GetKeyCode()
 		elif event.GetEventType() == wx.EVT_MENU.typeId:
 			keycode = self.id_to_keycode.get(event.GetId())
-		if keycode is not None:
-			#print chr(keycode)
+		if keycode == wx.WXK_TAB:
+			self.global_navigate() or event.Skip()
+		elif keycode >= 0:
 			if keycode == ord(" "):
-				if (event.GetEventType() != wx.EVT_MENU.typeId or
-					not isinstance(self.FindFocus(), FlatShadedButton) or
-					self.FindFocus() == self.adjustment_btn):
+				if (not isinstance(self.FindFocus(), FlatShadedButton) or
+					self.FindFocus() is self.adjustment_btn):
 					if self.is_measuring:
 						self.abort()
 					else:
 						self.start_interactive_adjustment()
+				else:
+					event.Skip()
 			elif keycode in [ord(str(c)) for c in range(1, 6)]:
 				key_num = chr(keycode)
 				pagenum = dict(zip(self.pagenum_2_argyll_key_num.values(),
@@ -1269,7 +1287,8 @@ class DisplayAdjustmentFrame(BaseFrame):
 					self.adjustment_btn.Enable()
 				self.is_busy = False
 				self.is_measuring = False
-				self.indicator_ctrl.SetBitmap(getbitmap("10x10/empty"))
+				self.indicator_ctrl.SetBitmap(geticon(10, "empty",
+													  use_mask=True))
 			self.calibration_btn.Enable()
 			self.lb.SetFocus()  # Make frame receive EVT_CHAR_HOOK events under Linux
 		elif "initial measurements" in txt or "check measurements" in txt:
