@@ -12,11 +12,12 @@ if sys.platform == "win32":
 
 from argyll_cgats import cal_to_fake_profile
 from argyll_names import video_encodings
-from config import (get_data_path, get_verified_path, getcfg, geticon, hascfg,
-					profile_ext, setcfg)
+from config import (defaults, get_data_path, get_verified_path, getcfg,
+					geticon, hascfg, profile_ext, setcfg)
 from log import safe_print
 from meta import name as appname, version
 from options import debug
+from util_decimal import stripzeros
 from util_os import islink, readlink, waccess
 from util_str import safe_unicode, strtr
 from worker import (Error, Info, UnloggedInfo, get_current_profile_path,
@@ -133,6 +134,15 @@ class LUT3DFrame(BaseFrame):
 		# Shared with main window
 		self.lut3d_apply_cal_cb.Bind(wx.EVT_CHECKBOX, self.lut3d_apply_cal_ctrl_handler)
 		self.lut3d_create_btn.Bind(wx.EVT_BUTTON, self.lut3d_create_handler)
+		self.lut3d_hdr_peak_luminance_ctrl.Bind(floatspin.EVT_FLOATSPIN,
+											self.lut3d_hdr_peak_luminance_handler)
+		self.lut3d_hdr_display_ctrl.Bind(wx.EVT_CHOICE, self.lut3d_hdr_display_handler)
+		self.lut3d_bind_trc_handlers()
+		self.lut3d_bind_hdr_trc_handlers()
+		self.lut3d_bind_content_colorspace_handlers()
+		self.lut3d_bind_common_handlers()
+
+	def lut3d_bind_trc_handlers(self):
 		self.lut3d_trc_ctrl.Bind(wx.EVT_CHOICE, self.lut3d_trc_ctrl_handler)
 		self.lut3d_trc_gamma_ctrl.Bind(wx.EVT_COMBOBOX,
 								 self.lut3d_trc_gamma_ctrl_handler)
@@ -144,11 +154,16 @@ class LUT3DFrame(BaseFrame):
 										   self.lut3d_trc_black_output_offset_ctrl_handler)
 		self.lut3d_trc_black_output_offset_intctrl.Bind(wx.EVT_TEXT,
 											  self.lut3d_trc_black_output_offset_ctrl_handler)
-		self.lut3d_hdr_peak_luminance_ctrl.Bind(wx.EVT_TEXT,
-											self.lut3d_hdr_peak_luminance_handler)
-		self.lut3d_hdr_display_ctrl.Bind(wx.EVT_CHOICE, self.lut3d_hdr_display_handler)
-		self.lut3d_hdr_maxcll_ctrl.Bind(wx.EVT_TEXT,
-										self.lut3d_hdr_maxcll_handler)
+
+	def lut3d_bind_hdr_trc_handlers(self):
+		self.lut3d_hdr_ambient_luminance_ctrl.Bind(floatspin.EVT_FLOATSPIN,
+												   self.lut3d_hdr_ambient_luminance_handler)
+		self.lut3d_hdr_minmll_ctrl.Bind(floatspin.EVT_FLOATSPIN,
+										self.lut3d_hdr_minmll_handler)
+		self.lut3d_hdr_maxmll_ctrl.Bind(floatspin.EVT_FLOATSPIN,
+										self.lut3d_hdr_maxmll_handler)
+
+	def lut3d_bind_content_colorspace_handlers(self):
 		self.lut3d_content_colorspace_ctrl.Bind(wx.EVT_CHOICE,
 												self.lut3d_content_colorspace_handler)
 		for color in ("white", "red", "green", "blue"):
@@ -157,6 +172,8 @@ class LUT3DFrame(BaseFrame):
 				getattr(self, "lut3d_content_colorspace_%s_%s" %
 							  (color, coord)).Bind(floatspin.EVT_FLOATSPIN,
 												   self.lut3d_content_colorspace_xy_handler)
+
+	def lut3d_bind_common_handlers(self):
 		self.encoding_input_ctrl.Bind(wx.EVT_CHOICE,
 											self.lut3d_encoding_input_ctrl_handler)
 		self.encoding_output_ctrl.Bind(wx.EVT_CHOICE,
@@ -196,7 +213,10 @@ class LUT3DFrame(BaseFrame):
 							options=("3dlut.", "last_3dlut_path",
 									 "position.lut3dframe", "size.lut3dframe"))
 		if event:
-			event.Skip()
+			# Hide first (looks nicer)
+			self.Hide()
+			# Need to use CallAfter to prevent hang under Windows if minimized
+			wx.CallAfter(self.Destroy)
 	
 	def use_abstract_profile_ctrl_handler(self, event):
 		setcfg("3dlut.use_abstract_profile",
@@ -217,9 +237,20 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_hdr_peak_luminance_label.Enable(v)
 		self.lut3d_hdr_peak_luminance_ctrl.Enable(v)
 		self.lut3d_hdr_peak_luminance_ctrl_label.Enable(v)
-		self.lut3d_hdr_maxcll_label.Enable(v)
-		self.lut3d_hdr_maxcll_ctrl.Enable(v)
-		self.lut3d_hdr_maxcll_ctrl_label.Enable(v)
+		self.lut3d_hdr_minmll_label.Enable(v)
+		self.lut3d_hdr_minmll_ctrl.Enable(v)
+		self.lut3d_hdr_minmll_ctrl_label.Enable(v)
+		self.lut3d_hdr_maxmll_label.Enable(v)
+		self.lut3d_hdr_maxmll_ctrl.Enable(v)
+		self.lut3d_hdr_maxmll_ctrl_label.Enable(v)
+		self.lut3d_hdr_diffuse_white_label.Enable(v)
+		self.lut3d_hdr_diffuse_white_txt.Enable(v)
+		self.lut3d_hdr_diffuse_white_txt_label.Enable(v)
+		self.lut3d_hdr_ambient_luminance_label.Enable(v)
+		self.lut3d_hdr_ambient_luminance_ctrl.Enable(v)
+		self.lut3d_hdr_ambient_luminance_ctrl_label.Enable(v)
+		self.lut3d_hdr_system_gamma_label.Enable(v)
+		self.lut3d_hdr_system_gamma_txt.Enable(v)
 		self.lut3d_content_colorspace_label.Enable(v)
 		self.lut3d_content_colorspace_ctrl.Enable(v)
 		for color in ("white", "red", "green", "blue"):
@@ -268,9 +299,17 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_set_option("3dlut.hdr_peak_luminance",
 							  self.lut3d_hdr_peak_luminance_ctrl.GetValue())
 
-	def lut3d_hdr_maxcll_handler(self, event):
-		self.lut3d_set_option("3dlut.hdr_maxcll",
-							  self.lut3d_hdr_maxcll_ctrl.GetValue())
+	def lut3d_hdr_ambient_luminance_handler(self, event):
+		self.lut3d_set_option("3dlut.hdr_ambient_luminance",
+							  self.lut3d_hdr_ambient_luminance_ctrl.GetValue())
+
+	def lut3d_hdr_minmll_handler(self, event):
+		self.lut3d_set_option("3dlut.hdr_minmll",
+							  self.lut3d_hdr_minmll_ctrl.GetValue())
+
+	def lut3d_hdr_maxmll_handler(self, event):
+		self.lut3d_set_option("3dlut.hdr_maxmll",
+							  self.lut3d_hdr_maxmll_ctrl.GetValue())
 
 	def lut3d_trc_black_output_offset_ctrl_handler(self, event):
 		if event.GetId() == self.lut3d_trc_black_output_offset_intctrl.GetId():
@@ -283,7 +322,7 @@ class LUT3DFrame(BaseFrame):
 		if v != getcfg("3dlut.trc_output_offset"):
 			self.lut3d_set_option("3dlut.trc_output_offset", v)
 			self.lut3d_update_trc_control()
-			self.lut3d_show_trc_controls()
+			#self.lut3d_show_trc_controls()
 
 	def lut3d_trc_gamma_ctrl_handler(self, event):
 		try:
@@ -300,7 +339,7 @@ class LUT3DFrame(BaseFrame):
 			if v != getcfg("3dlut.trc_gamma"):
 				self.lut3d_set_option("3dlut.trc_gamma", v)
 				self.lut3d_update_trc_control()
-				self.lut3d_show_trc_controls()
+				#self.lut3d_show_trc_controls()
 		event.Skip()
 
 	def lut3d_trc_ctrl_handler(self, event):
@@ -320,13 +359,20 @@ class LUT3DFrame(BaseFrame):
 		elif self.lut3d_trc_ctrl.GetSelection() == 2:  # SMPTE 2084, hard clip
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
 			trc = "smpte2084.hardclip"
-			self.lut3d_set_option("3dlut.hdr_maxcll", 10000)
+			self.lut3d_set_option("3dlut.hdr_maxmll", 10000)
 		elif self.lut3d_trc_ctrl.GetSelection() == 3:  # SMPTE 2084, roll-off clip
 			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
 			trc = "smpte2084.rolloffclip"
-			self.lut3d_set_option("3dlut.hdr_maxcll", 10000)
+			self.lut3d_set_option("3dlut.hdr_maxmll", 10000)
+		elif self.lut3d_trc_ctrl.GetSelection() == 4:  # HLG
+			self.lut3d_set_option("3dlut.trc_output_offset", 0.0)
+			trc = "hlg"
 		else:
 			trc = "customgamma"
+			# Have to use CallAfter, otherwise only part of the text will
+			# be selected (wxPython bug?)
+			wx.CallAfter(self.lut3d_trc_gamma_ctrl.SetFocus)
+			wx.CallLater(1, self.lut3d_trc_gamma_ctrl.SelectAll)
 		self.lut3d_set_option("3dlut.trc", trc)
 		if trc != "customgamma":
 			self.lut3d_update_trc_controls()
@@ -348,11 +394,6 @@ class LUT3DFrame(BaseFrame):
 	def lut3d_encoding_input_ctrl_handler(self, event):
 		encoding = self.encoding_input_ab[self.encoding_input_ctrl.GetSelection()]
 		self.lut3d_set_option("3dlut.encoding.input", encoding)
-		if getcfg("3dlut.format") == "eeColor":
-			if encoding == "T":
-				encoding = "t"
-			self.encoding_output_ctrl.SetSelection(self.encoding_output_ba[encoding])
-			self.lut3d_set_option("3dlut.encoding.output", encoding)
 		if getattr(self, "lut3dframe", None):
 			self.lut3dframe.lut3d_update_encoding_controls()
 		elif self.Parent:
@@ -713,39 +754,44 @@ class LUT3DFrame(BaseFrame):
 											 "${WIDTH}": str(clut_size ** 2),
 											 "${HEIGHT}": str(clut_size),
 											 "${FORMAT}": "RGBA%i" % getcfg("3dlut.bitdepth.output")})
-							reshade_fx_path = os.path.join(dst_dir, "ReShade.fx")
-							# Adjust path for correct installation if ReShade.fx
-							# is a symlink.
-							if islink(reshade_fx_path):
-								reshade_fx_path = readlink(reshade_fx_path)
-								path = os.path.join(os.path.dirname(reshade_fx_path),
+							reshade_shaders = os.path.join(dst_dir,
+														   "reshade-shaders")
+							if os.path.isdir(reshade_shaders):
+								# ReShade >= 3.x with default shaders
+								path = os.path.join(reshade_shaders, "Textures",
 													os.path.basename(path))
 								dst_paths = [path]
-							if os.path.isfile(reshade_fx_path):
-								# Alter existing ReShade.fx
-								with open(reshade_fx_path, "rb") as reshade_fx_file:
-									reshade_fx = reshade_fx_file.read()
-								# Remove existing shader include
-								reshade_fx = re.sub(r'[ \t]*//\s*Automatically\s+\S+\s+by\s+%s\s+.+[ \t]*\r?\n?' %
-													appname, "", reshade_fx)
-								reshade_fx = re.sub(r'[ \t]*#include\s+"ColorLookupTable.fx"[ \t]*\r?\n?',
-												    "", reshade_fx).rstrip("\r\n")
+								dst_dir = os.path.join(reshade_shaders,
+													   "Shaders")
 							else:
-								reshade_fx = ""
-							if reshade_fx:
-								reshade_fx += "%s// Automatically added by %s %s%s" % (os.linesep * 2, appname, version, os.linesep)
-							else:
-								reshade_fx = "// Automatically created by %s %s%s" % (appname, version, os.linesep)
-							reshade_fx += '#include "ColorLookupTable.fx"' + os.linesep
-							clut_fx_path = os.path.join(os.path.dirname(path),
+								reshade_fx_path = os.path.join(dst_dir, "ReShade.fx")
+								# Adjust path for correct installation if ReShade.fx
+								# is a symlink.
+								if islink(reshade_fx_path):
+									# ReShade < 3.0
+									reshade_fx_path = readlink(reshade_fx_path)
+									dst_dir = os.path.dirname(reshade_fx_path)
+									path = os.path.join(dst_dir,
+														os.path.basename(path))
+									dst_paths = [path]
+								if os.path.isfile(reshade_fx_path):
+									# ReShade < 3.0
+									# Alter existing ReShade.fx
+									with open(reshade_fx_path, "rb") as reshade_fx_file:
+										reshade_fx = reshade_fx_file.read()
+									# Remove existing shader include
+									reshade_fx = re.sub(r'[ \t]*//\s*Automatically\s+\S+\s+by\s+%s\s+.+[ \t]*\r?\n?' %
+														appname, "", reshade_fx)
+									reshade_fx = re.sub(r'[ \t]*#include\s+"ColorLookupTable.fx"[ \t]*\r?\n?',
+														"", reshade_fx).rstrip("\r\n")
+									reshade_fx += "%s// Automatically added by %s %s%s" % (os.linesep * 2, appname, version, os.linesep)
+									reshade_fx += '#include "ColorLookupTable.fx"' + os.linesep
+									with open(reshade_fx_path, "wb") as reshade_fx_file:
+										reshade_fx_file.write(reshade_fx)
+							clut_fx_path = os.path.join(dst_dir,
 														"ColorLookupTable.fx")
 							with open(clut_fx_path, "wb") as clut_fx_file:
 								clut_fx_file.write(clut_fx)
-							if not os.path.isdir(os.path.join(dst_dir,
-															  "reshade-shaders")):
-								# ReShade < 3.0
-								with open(reshade_fx_path, "wb") as reshade_fx_file:
-									reshade_fx_file.write(reshade_fx)
 					for src_path, dst_path in zip(src_paths, dst_paths):
 						shutil.copyfile(src_path, dst_path)
 					return
@@ -767,7 +813,8 @@ class LUT3DFrame(BaseFrame):
 		output_encoding = getcfg("3dlut.encoding.output")
 		if (getcfg("3dlut.apply_trc") or
 			not hasattr(self, "lut3d_trc_apply_none_ctrl")):
-			if getcfg("3dlut.trc").startswith("smpte2084"):
+			if (getcfg("3dlut.trc").startswith("smpte2084") or  # SMPTE ST.2084 (PQ)
+				getcfg("3dlut.trc") == "hlg"):  # Hybrid Log-Gamma (HLG)
 				trc_gamma = getcfg("3dlut.trc")
 			else:
 				trc_gamma = getcfg("3dlut.trc_gamma")
@@ -789,8 +836,10 @@ class LUT3DFrame(BaseFrame):
 		apply_black_offset = getcfg("3dlut.apply_black_offset")
 		use_b2a = getcfg("3dlut.gamap.use_b2a")
 		white_cdm2 = getcfg("3dlut.hdr_peak_luminance")
-		maxcll = getcfg("3dlut.hdr_maxcll")
-		content_rgb_space = [-2084, [], [], [], []]
+		minmll = getcfg("3dlut.hdr_minmll")
+		maxmll = getcfg("3dlut.hdr_maxmll")
+		ambient_cdm2 = getcfg("3dlut.hdr_ambient_luminance")
+		content_rgb_space = [1.0, [], [], [], []]
 		for i, color in enumerate(("white", "red", "green", "blue")):
 			for coord in "xy":
 				v = getcfg("3dlut.content.colorspace.%s.%s" % (color, coord))
@@ -812,7 +861,8 @@ class LUT3DFrame(BaseFrame):
 									 trc_output_offset=outoffset,
 									 apply_black_offset=apply_black_offset,
 									 use_b2a=use_b2a, white_cdm2=white_cdm2,
-									 maxcll=maxcll,
+									 minmll=minmll, maxmll=maxmll,
+									 ambient_cdm2=ambient_cdm2,
 									 content_rgb_space=content_rgb_space,
 									 hdr_display=getcfg("3dlut.hdr_display"),
 									 XYZwp=XYZwp)
@@ -966,7 +1016,7 @@ class LUT3DFrame(BaseFrame):
 		if getattr(self, "lut3dframe", None):
 			self.lut3dframe.lut3d_update_shared_controls()
 		else:
-			if hasattr(self, "lut3d_show_input_value_clipping_warning"):
+			if isinstance(self, LUT3DFrame):
 				self.lut3d_show_input_value_clipping_warning(True)
 			if self.Parent:
 				self.Parent.lut3d_update_shared_controls()
@@ -1003,9 +1053,9 @@ class LUT3DFrame(BaseFrame):
 					(which == "abstract" and
 					 (profile.profileClass != "abst" or profile.colorSpace
 					  not in ("Lab", "XYZ")))):
-					show_result_dialog(NotImplementedError(lang.getstr("profile.unsupported", 
-																	   (profile.profileClass, 
-																		profile.colorSpace))),
+					show_result_dialog(Error(lang.getstr("profile.unsupported", 
+														 (profile.profileClass, 
+														  profile.colorSpace))),
 									   parent=self)
 				else:
 					if profile.profileClass == "link":
@@ -1085,6 +1135,7 @@ class LUT3DFrame(BaseFrame):
 							setattr(self, "input_profile", profile)
 							if not self.set_profile("output", silent=silent):
 								self.update_linking_controls()
+								self.lut3d_trc_apply_ctrl_handler()
 						elif which == "output":
 							if (not hasattr(self, which + "_profile") or
 								getcfg("3dlut.%s.profile" % which) !=
@@ -1177,7 +1228,7 @@ class LUT3DFrame(BaseFrame):
 		# Shared with main window
 		items = []
 		for item in ("Gamma 2.2", "trc.rec1886", "trc.smpte2084.hardclip",
-					 "trc.smpte2084.rolloffclip", "custom"):
+					 "trc.smpte2084.rolloffclip", "trc.hlg", "custom"):
 			items.append(lang.getstr(item))
 		self.lut3d_trc_ctrl.SetItems(items)
 		
@@ -1246,12 +1297,6 @@ class LUT3DFrame(BaseFrame):
 			encodings = list(video_encodings)
 			config.defaults["3dlut.encoding.input"] = "n"
 			config.defaults["3dlut.encoding.output"] = "n"
-			if format == "eeColor":
-				# As eeColor usually needs same input & output encoding,
-				# and xvYCC output encoding is not supported generally,
-				# remove xvYCC input encoding choices for eeColor
-				encodings.remove("x")
-				encodings.remove("X")
 		if (self.worker.argyll_version >= [1, 7] and
 			self.worker.argyll_version != [1, 7, 0, "_beta"]):
 			# Argyll 1.7 beta 3 (2015-04-02) added clip WTW on input TV encoding
@@ -1288,6 +1333,35 @@ class LUT3DFrame(BaseFrame):
 			getcfg("3dlut.create") and v != getcfg(option)):
 			self.profile_settings_changed()
 		setcfg(option, v)
+		if option in ("3dlut.hdr_peak_luminance", "3dlut.hdr_minmll",
+					  "3dlut.hdr_maxmll"):
+			self.lut3d_hdr_update_diffuse_white()
+		elif option == "3dlut.hdr_ambient_luminance":
+			self.lut3d_hdr_update_system_gamma()
+
+	def lut3d_hdr_update_diffuse_white(self):
+		# Update knee start info for BT.2390-3 roll-off
+		bt2390 = colormath.BT2390(0, getcfg("3dlut.hdr_peak_luminance"),
+								  getcfg("3dlut.hdr_minmll"),
+								  getcfg("3dlut.hdr_maxmll"))
+		diffuse_ref_cdm2 = 94.37844
+		diffuse_PQ = colormath.specialpow(diffuse_ref_cdm2 / 10000, 1.0 / -2084)
+		# Determine white cd/m2 after roll-off
+		diffuse_tgt_cdm2 = colormath.specialpow(bt2390.apply(diffuse_PQ),
+												  -2084) * 10000
+		if diffuse_tgt_cdm2 < diffuse_ref_cdm2:
+			signalcolor = "#CC0000"
+		else:
+			signalcolor = "#008000"
+		self.lut3d_hdr_diffuse_white_txt.ForegroundColour = signalcolor
+		self.lut3d_hdr_diffuse_white_txt.Label = "%.2f" % diffuse_tgt_cdm2
+		self.lut3d_hdr_diffuse_white_txt_label.ForegroundColour = signalcolor
+		self.lut3d_hdr_diffuse_white_txt.ContainingSizer.Layout()
+
+	def lut3d_hdr_update_system_gamma(self):
+		# Update system gamma for HLG based on ambient luminance (BT.2390-3)
+		hlg = colormath.HLG(ambient_cdm2=getcfg("3dlut.hdr_ambient_luminance"))
+		self.lut3d_hdr_system_gamma_txt.Label = str(stripzeros("%.4f" % hlg.gamma))
 	
 	def update_controls(self):
 		""" Update controls with values from the configuration """
@@ -1308,12 +1382,9 @@ class LUT3DFrame(BaseFrame):
 		# Shared with main window
 		self.lut3d_update_trc_controls()
 		self.lut3d_rendering_intent_ctrl.SetSelection(self.rendering_intents_ba[getcfg("3dlut.rendering_intent")])
-		format = getcfg("3dlut.format")
-		if format == "madVR" and self.worker.argyll_version < [1, 6]:
-			# MadVR only available with Argyll 1.6+, fall back to IRIDAS .cube
-			format = "cube"
-			setcfg("3dlut.format", format)
-		self.lut3d_format_ctrl.SetSelection(self.lut3d_formats_ba[getcfg("3dlut.format")])
+		# MadVR only available with Argyll 1.6+, fall back to default
+		self.lut3d_format_ctrl.SetSelection(self.lut3d_formats_ba.get(getcfg("3dlut.format"),
+																	  self.lut3d_formats_ba[defaults["3dlut.format"]]))
 		self.lut3d_hdr_display_ctrl.SetSelection(getcfg("3dlut.hdr_display"))
 		self.lut3d_size_ctrl.SetSelection(self.lut3d_size_ba[getcfg("3dlut.size")])
 		self.lut3d_enable_size_controls()
@@ -1330,6 +1401,8 @@ class LUT3DFrame(BaseFrame):
 			else:
 				sel = 3
 			self.lut3d_trc_ctrl.SetSelection(sel)
+		elif getcfg("3dlut.trc") == "hlg":  # Hybrid Log-Gamma (HLG)
+			self.lut3d_trc_ctrl.SetSelection(4)
 		elif (getcfg("3dlut.trc_gamma_type") == "B" and
 			getcfg("3dlut.trc_output_offset") == 0 and
 			getcfg("3dlut.trc_gamma") == 2.4):
@@ -1341,7 +1414,7 @@ class LUT3DFrame(BaseFrame):
 			self.lut3d_trc_ctrl.SetSelection(0)  # Pure power gamma 2.2
 			setcfg("3dlut.trc", "gamma2.2")
 		else:
-			self.lut3d_trc_ctrl.SetSelection(4)  # Custom
+			self.lut3d_trc_ctrl.SetSelection(5)  # Custom
 			setcfg("3dlut.trc", "customgamma")
 
 	def lut3d_update_trc_controls(self):
@@ -1352,25 +1425,23 @@ class LUT3DFrame(BaseFrame):
 		self.lut3d_trc_black_output_offset_ctrl.SetValue(outoffset)
 		self.lut3d_trc_black_output_offset_intctrl.SetValue(outoffset)
 		self.lut3d_hdr_peak_luminance_ctrl.SetValue(getcfg("3dlut.hdr_peak_luminance"))
-		self.lut3d_hdr_maxcll_ctrl.SetValue(getcfg("3dlut.hdr_maxcll"))
+		self.lut3d_hdr_minmll_ctrl.SetValue(getcfg("3dlut.hdr_minmll"))
+		self.lut3d_hdr_maxmll_ctrl.SetValue(getcfg("3dlut.hdr_maxmll"))
+		self.lut3d_hdr_update_diffuse_white()
+		self.lut3d_hdr_ambient_luminance_ctrl.SetValue(getcfg("3dlut.hdr_ambient_luminance"))
+		self.lut3d_hdr_update_system_gamma()
 		# Content colorspace (currently only used for SMPTE 2084)
 		content_colors = []
-		for color in ("white", "red", "green", "blue"):
+		for color in ("red", "green", "blue", "white"):
 			for coord in "xy":
 				v = getcfg("3dlut.content.colorspace.%s.%s" % (color, coord))
 				getattr(self, "lut3d_content_colorspace_%s_%s" %
 							  (color, coord)).SetValue(v)
 				content_colors.append(round(v, 4))
-		for i, rgb_space in enumerate(self.lut3d_content_colorspace_names):
-			rgb_space = colormath.get_rgb_space(rgb_space)
-			wx, wy = (round(v, 4) for v in
-					  colormath.XYZ2xyY(*colormath.get_whitepoint(rgb_space[1]))[:2])
-			colors = [wx, wy]
-			for primary in rgb_space[2:5]:
-				for j in xrange(2):
-					colors.append(round(primary[j], 4))
-			if colors == content_colors:
-				break
+		rgb_space_name = colormath.find_primaries_wp_xy_rgb_space_name(content_colors,
+																	   self.lut3d_content_colorspace_names)
+		if rgb_space_name:
+			i = self.lut3d_content_colorspace_names.index(rgb_space_name)
 		else:
 			i = self.lut3d_content_colorspace_ctrl.Count - 1
 		self.lut3d_content_colorspace_ctrl.SetSelection(i)
@@ -1390,7 +1461,7 @@ class LUT3DFrame(BaseFrame):
 			self.input_profile.tags.bTRC and
 			isinstance(self.input_profile.tags.rTRC,
 					   ICCP.CurveType)):
-			tf = self.input_profile.tags.rTRC.get_transfer_function()
+			tf = self.input_profile.tags.rTRC.get_transfer_function(outoffset=1.0)
 			if (getcfg("3dlut.input.profile") !=
 				self.input_profile.fileName):
 				# Use BT.1886 gamma mapping for SMPTE 240M /
@@ -1475,27 +1546,40 @@ class LUT3DFrame(BaseFrame):
 			self.lut3d_trc_apply_ctrl.Show(show)
 		self.lut3d_trc_ctrl.Show(show)
 		smpte2084 = getcfg("3dlut.trc").startswith("smpte2084")
+		hlg = getcfg("3dlut.trc") == "hlg"
+		hdr = smpte2084 or hlg
 		show = show and (getcfg("3dlut.trc") == "customgamma" or
 						 (isinstance(self, LUT3DFrame) or
 						  getcfg("show_advanced_options")))
-		self.lut3d_trc_gamma_label.Show(show and not smpte2084)
-		self.lut3d_trc_gamma_ctrl.Show(show and not smpte2084)
+		self.lut3d_trc_gamma_label.Show(show and not hdr)
+		self.lut3d_trc_gamma_ctrl.Show(show and not hdr)
 		smpte2084r = getcfg("3dlut.trc") == "smpte2084.rolloffclip"
 		# Show items in this order so we end up with the correct controls shown
-		showcc = smpte2084r and (isinstance(self, LUT3DFrame) or
-								 getcfg("show_advanced_options"))
+		showcc = (smpte2084r or hlg) and (isinstance(self, LUT3DFrame) or
+										  getcfg("show_advanced_options"))
 		self.lut3d_content_colorspace_label.ContainingSizer.ShowItems(showcc)
 		sel = self.lut3d_content_colorspace_ctrl.Selection
 		lastsel = self.lut3d_content_colorspace_ctrl.Count - 1
 		sizer = self.lut3d_content_colorspace_red_x.ContainingSizer
 		sizer.ShowItems(showcc and sel == lastsel)
-		self.lut3d_hdr_maxcll_label.Show(show and smpte2084r)
-		self.lut3d_hdr_maxcll_ctrl.Show(show and smpte2084r)
-		self.lut3d_hdr_maxcll_ctrl_label.Show(show and smpte2084r)
-		show = show or smpte2084
+		self.lut3d_hdr_minmll_label.Show(show and smpte2084)
+		self.lut3d_hdr_minmll_ctrl.Show(show and smpte2084)
+		self.lut3d_hdr_minmll_ctrl_label.Show(show and smpte2084)
+		self.lut3d_hdr_maxmll_label.Show(show and smpte2084r)
+		self.lut3d_hdr_maxmll_ctrl.Show(show and smpte2084r)
+		self.lut3d_hdr_maxmll_ctrl_label.Show(show and smpte2084r)
+		self.lut3d_hdr_diffuse_white_label.Show(show and smpte2084r)
+		self.lut3d_hdr_diffuse_white_txt.Show(show and smpte2084r)
+		self.lut3d_hdr_diffuse_white_txt_label.Show(show and smpte2084r)
+		self.lut3d_hdr_ambient_luminance_label.Show(show and hlg)
+		self.lut3d_hdr_ambient_luminance_ctrl.Show(show and hlg)
+		self.lut3d_hdr_ambient_luminance_ctrl_label.Show(show and hlg)
+		self.lut3d_hdr_system_gamma_label.Show(show and hlg)
+		self.lut3d_hdr_system_gamma_txt.Show(show and hlg)
+		show = (show or smpte2084) and not hlg
 		show = show and ((hasattr(self, "lut3d_create_cb") and
 						  getcfg("3dlut.create")) or self.XYZbpout > [0, 0, 0])
-		self.lut3d_trc_gamma_type_ctrl.Show(show and not smpte2084)
+		self.lut3d_trc_gamma_type_ctrl.Show(show and not hdr)
 		self.lut3d_trc_black_output_offset_label.Show(show)
 		self.lut3d_trc_black_output_offset_ctrl.Show(show)
 		self.lut3d_trc_black_output_offset_intctrl.Show(show)
@@ -1525,8 +1609,7 @@ class LUT3DFrame(BaseFrame):
 		self.encoding_input_ctrl.SetSelection(self.encoding_input_ba[getcfg("3dlut.encoding.input")])
 		self.encoding_input_ctrl.Enable(self.encoding_input_ctrl.Count > 1)
 		self.encoding_output_ctrl.SetSelection(self.encoding_output_ba[getcfg("3dlut.encoding.output")])
-		self.encoding_output_ctrl.Enable(getcfg("3dlut.format") not in ("eeColor",
-																		"madVR"))
+		self.encoding_output_ctrl.Enable(getcfg("3dlut.format") != "madVR")
 	
 	def lut3d_enable_size_controls(self):
 		self.lut3d_size_ctrl.Enable(getcfg("3dlut.format")
