@@ -7,8 +7,9 @@ import subprocess as sp
 import sys
 import tempfile
 
+import numpy
+
 from argyll_cgats import cal_to_fake_profile, vcgt_to_cal
-from colormath import interp
 from config import (fs_enc, get_argyll_display_number, get_data_path,
 					get_display_profile, get_display_rects, getcfg, geticon,
 					get_verified_path, setcfg)
@@ -80,7 +81,8 @@ class CoordinateType(list):
 				vmax = self[-1][0]
 		return colormath.get_gamma(values, 100.0, vmin, vmax, average, least_squares)
 	
-	def get_transfer_function(self, best=True, slice=(0.05, 0.95)):
+	def get_transfer_function(self, best=True, slice=(0.05, 0.95),
+							  outoffset=None):
 		"""
 		Return transfer function name, exponent and match percentage
 		
@@ -88,66 +90,18 @@ class CoordinateType(list):
 		transfer_function = self._transfer_function.get((best, slice))
 		if transfer_function:
 			return transfer_function
-		trc = ICCP.CurveType()
-		match = {}
-		vmin = self[0][0]
-		vmax = self[-1][0]
-		best_yx = (0, 255)
-		for i, (y, x) in enumerate(self):
-			if x - 127.5 > 0 and x < best_yx[1]:
-				best_yx = (y, x)
-		gamma = colormath.get_gamma([(best_yx[1] / 255.0 * 100.0, best_yx[0])], 100.0, vmin, vmax)
-		for name, exp in (("Rec. 709", -709),
-						  ("Rec. 1886", -1886),
-						  ("SMPTE 240M", -240),
-						  ("SMPTE 2084", -2084),
-						  ("DICOM", -1023),
-						  ("L*", -3.0),
-						  ("sRGB", -2.4),
-						  ("Gamma %.2f" % gamma, gamma)):
-			if name in ("DICOM", "Rec. 1886", "SMPTE 2084"):
-				if self.profile and isinstance(self.profile.tags.get("lumi"),
-											   ICCP.XYZType):
-					white_cdm2 = self.profile.tags.lumi.Y
-				else:
-					white_cdm2 = 100.0
-				black_Y = vmin / 100.0
-				black_cdm2 = black_Y * white_cdm2
-				try:
-					if name == "DICOM":
-						trc.set_dicom_trc(black_cdm2, white_cdm2, size=len(self))
-					elif name == "Rec. 1886":
-						trc.set_bt1886_trc(black_Y, size=len(self))
-					elif name == "SMPTE 2084":
-						trc.set_smpte2084_trc(black_cdm2, white_cdm2, size=len(self))
-				except ValueError:
-					continue
-			else:
-				trc.set_trc(exp, len(self), vmin * 655.35, vmax * 655.35)
-			match[(name, exp)] = 0.0
-			count = 0
-			start = slice[0] * 255
-			end = slice[1] * 255
-			for i, (n, x) in enumerate(self):
-				##n = colormath.XYZ2Lab(0, n, 0)[0]
-				if x >= start and x <= end:
-					n = colormath.get_gamma([(x / 255.0 * 100, n)], 100.0, vmin, vmax, False)
-					if n:
-						n = n[0]
-						##n2 = colormath.XYZ2Lab(0, trc[i][0], 0)[0]
-						n2 = colormath.get_gamma([(i / (len(trc) - 1.0) * 65535, trc[i])], 65535, vmin * 655.35, vmax * 655.35, False)
-						if n2 and n2[0]:
-							n2 = n2[0]
-							match[(name, exp)] += 1 - (max(n, n2) - min(n, n2)) / n2
-							count += 1
-			if count:
-				match[(name, exp)] /= count
-		if not best:
-			self._transfer_function[(best, slice)] = match
-			return match
-		match, (name, exp) = sorted(zip(match.values(), match.keys()))[-1]
-		self._transfer_function[(best, slice)] = (name, exp), match
-		return (name, exp), match
+		xp = []
+		fp = []
+		for y, x in self:
+			xp.append(x)
+			fp.append(y)
+		interp = colormath.Interp(xp, fp, use_numpy=True)
+		otrc = ICCP.CurveType(profile=self.profile)
+		for i in xrange(len(self)):
+			otrc.append(interp(i / (len(self) - 1.0) * 255) / 100 * 65535)
+		match = otrc.get_transfer_function(best, slice, outoffset=outoffset)
+		self._transfer_function[(best, slice)] = match
+		return match
 	
 	def set_trc(self, power=2.2, values=(), vmin=0, vmax=100):
 		"""
@@ -754,7 +708,7 @@ class LUTFrame(BaseFrame):
 				  id=self.plot_mode_select.GetId())
 		
 		self.tooltip_btn = BitmapButton(panel, -1,
-										geticon(16, "dialog-information"),
+										geticon(16, "question-inverted"),
 										style=wx.NO_BORDER)
 		self.tooltip_btn.SetBackgroundColour(BGCOLOUR)
 		self.tooltip_btn.Bind(wx.EVT_BUTTON, self.tooltip_handler)
@@ -762,7 +716,7 @@ class LUTFrame(BaseFrame):
 		panel.Sizer.Add(self.tooltip_btn, flag=wx.ALIGN_CENTER_VERTICAL)
 
 		self.save_plot_btn = BitmapButton(panel, -1,
-										  geticon(16, "camera-photo"),
+										  geticon(16, "image-x-generic-inverted"),
 										  style=wx.NO_BORDER)
 		self.save_plot_btn.SetBackgroundColour(BGCOLOUR)
 		panel.Sizer.Add(self.save_plot_btn, flag=wx.ALIGN_CENTER_VERTICAL)
@@ -850,7 +804,7 @@ class LUTFrame(BaseFrame):
 		self.cbox_sizer.Add((10, 0))
 
 		self.reload_vcgt_btn = BitmapButton(self.box_panel, -1,
-											geticon(16, "stock_refresh"),
+											geticon(16, "stock_refresh-inverted"),
 											style=wx.NO_BORDER)
 		self.reload_vcgt_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.reload_vcgt_btn, flag=wx.ALIGN_CENTER_VERTICAL |
@@ -861,7 +815,7 @@ class LUTFrame(BaseFrame):
 		self.reload_vcgt_btn.Disable()
 		
 		self.apply_bpc_btn = BitmapButton(self.box_panel, -1,
-										  geticon(16, "black_point"),
+										  geticon(16, "color-inverted"),
 										  style=wx.NO_BORDER)
 		self.apply_bpc_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.apply_bpc_btn, flag=wx.ALIGN_CENTER_VERTICAL |
@@ -871,7 +825,7 @@ class LUTFrame(BaseFrame):
 		self.apply_bpc_btn.Disable()
 
 		self.install_vcgt_btn = BitmapButton(self.box_panel, -1,
-											 geticon(16, "install"),
+											 geticon(16, "install-inverted"),
 											 style=wx.NO_BORDER)
 		self.install_vcgt_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.install_vcgt_btn,
@@ -881,7 +835,7 @@ class LUTFrame(BaseFrame):
 		self.install_vcgt_btn.Disable()
 
 		self.save_vcgt_btn = BitmapButton(self.box_panel, -1,
-										  geticon(16, "media-floppy"),
+										  geticon(16, "document-save-as-inverted"),
 										  style=wx.NO_BORDER)
 		self.save_vcgt_btn.SetBackgroundColour(BGCOLOUR)
 		self.cbox_sizer.Add(self.save_vcgt_btn, flag=wx.ALIGN_CENTER_VERTICAL |
@@ -1173,7 +1127,7 @@ class LUTFrame(BaseFrame):
 		if profile.connectionColorSpace == "RGB":
 			mult = 1
 		else:
-			mult = 2
+			mult = 4
 		size = 256 * mult  # Final number of coordinates
 
 		if hasattr(self, "rendering_intent_select"):
@@ -1278,13 +1232,15 @@ class LUTFrame(BaseFrame):
 
 		if direction in ("b", "if") or profile.connectionColorSpace == "RGB":
 			if direction == "if":
+				Lbp = self.worker.xicclu(profile, [[0, 0, 0]], intent, "f",
+										 order, "l", use_icclu=use_icclu)[0][0]
 				maxval = size - 1.0
 
 				# Deal with values that got clipped (below black as well as white)
 				do_low_clip = True
 				for i, values in enumerate(odata):
-					if values[3] is True:
-						if do_low_clip:
+					if values[3] is True or i == 0:
+						if do_low_clip and (i / maxval * 100 < Lbp or i == 0):
 							# Set to black
 							values[:] = [0.0, 0.0, 0.0]
 						elif (i == maxval and
@@ -1354,8 +1310,8 @@ class LUTFrame(BaseFrame):
 			for i in xrange(size):
 				x.append(i / (size - 1.0) * 255)
 				y.append(colormath.Lab2XYZ(i / (size - 1.0) * 100, 0, 0)[1] * 100)
-			xi = interp(y, yp, xp)
-			yi = interp(x, xi, y)
+			xi = numpy.interp(y, yp, xp)
+			yi = numpy.interp(x, xi, y)
 			setattr(self, "tf_" + sig, CoordinateType(self.profile))
 			for Y, v in zip(yi, x):
 				if Y <= yp[0]:
@@ -1378,7 +1334,7 @@ class LUTFrame(BaseFrame):
 		event.Skip()
 	
 	def plot_mode_select_handler(self, event):
-		self.client.resetzoom()
+		#self.client.resetzoom()
 		self.DrawLUT()
 
 	def get_commands(self):
@@ -1459,6 +1415,25 @@ class LUTFrame(BaseFrame):
 				wx.CallAfter(show_result_dialog, exception, self)
 			else:
 				curves.append(lang.getstr('[rgb]TRC'))
+		if getcfg("show_advanced_options"):
+			if isinstance(self.profile.tags.get("A2B0"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.A2B0.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.A2B0.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("A2B1"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.A2B1.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.A2B1.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("A2B2"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.A2B2.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.A2B2.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A0"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.B2A0.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.B2A0.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A1"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.B2A1.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.B2A1.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A2"), ICCP.LUT16Type):
+				curves.append(lang.getstr('profile.tags.B2A2.shaper_curves.input'))
+				curves.append(lang.getstr('profile.tags.B2A2.shaper_curves.output'))
 		selection = self.plot_mode_select.GetSelection()
 		center = False
 		if curves and (selection < 0 or selection > len(curves) - 1):
@@ -1551,7 +1526,8 @@ class LUTFrame(BaseFrame):
 										  self.tf_gTRC[i][1] +
 										  self.tf_bTRC[i][1]) / 3.0])
 			if getattr(self, "trc", None):
-				transfer_function = self.trc.get_transfer_function(slice=(0.00, 1.00))
+				transfer_function = self.trc.get_transfer_function(slice=(0.00, 1.00),
+																   outoffset=1.0)
 			#if "R" in colorants and "G" in colorants and "B" in colorants:
 				#if self.profile.tags.rTRC == self.profile.tags.gTRC == self.profile.tags.bTRC:
 					#transfer_function = self.profile.tags.rTRC.get_transfer_function()
@@ -1631,6 +1607,40 @@ class LUTFrame(BaseFrame):
 						'entryCount': len(self.rTRC),
 						'entrySize': 2
 					}
+			elif self.plot_mode_select.GetSelection() in range(2, 14):
+				if self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B0.shaper_curves.input'):
+					tables = self.profile.tags.A2B0.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B0.shaper_curves.output'):
+					tables = self.profile.tags.A2B0.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B1.shaper_curves.input'):
+					tables = self.profile.tags.A2B1.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B1.shaper_curves.output'):
+					tables = self.profile.tags.A2B1.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B2.shaper_curves.input'):
+					tables = self.profile.tags.A2B2.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.A2B2.shaper_curves.output'):
+					tables = self.profile.tags.A2B2.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A0.shaper_curves.input'):
+					tables = self.profile.tags.B2A0.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A0.shaper_curves.output'):
+					tables = self.profile.tags.B2A0.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A1.shaper_curves.input'):
+					tables = self.profile.tags.B2A1.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A1.shaper_curves.output'):
+					tables = self.profile.tags.B2A1.output
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A2.shaper_curves.input'):
+					tables = self.profile.tags.B2A2.input
+				elif self.plot_mode_select.GetStringSelection() == lang.getstr('profile.tags.B2A2.shaper_curves.output'):
+					tables = self.profile.tags.B2A2.output
+				entry_count = len(tables[0])
+				x = [v / (entry_count - 1.0) * 255 for v in range(entry_count)]
+				curves = {
+					'data': [zip(x, [v / 65535.0 * 255 for v in tables[0]]),
+							 zip(x, [v / 65535.0 * 255 for v in tables[1]]),
+							 zip(x, [v / 65535.0 * 255 for v in tables[2]])],
+					'entryCount': entry_count,
+					'entrySize': 2
+				}
 		yLabel = []
 		if self.toggle_red.GetValue():
 			yLabel.append("R")
@@ -1638,7 +1648,7 @@ class LUTFrame(BaseFrame):
 			yLabel.append("G")
 		if self.toggle_blue.GetValue():
 			yLabel.append("B")
-		if (self.plot_mode_select.GetSelection() == 0 or
+		if (self.plot_mode_select.GetSelection() in [0] + range(2, 14) or
 			(self.profile and self.profile.connectionColorSpace == "RGB")):
 			self.xLabel = "".join(yLabel)
 		else:
@@ -1652,7 +1662,7 @@ class LUTFrame(BaseFrame):
 		self.toggle_green.Enable(bool(curves))
 		self.toggle_blue.Enable(bool(curves))
 		self.show_as_L.Enable(bool(curves))
-		self.show_as_L.Show(self.plot_mode_select.GetSelection() != 0 and
+		self.show_as_L.Show(self.plot_mode_select.GetSelection() not in [0] + range(2, 14) and
 							self.profile.connectionColorSpace != "RGB")
 		self.toggle_clut.Show(self.profile.connectionColorSpace != "RGB" and
 							  self.plot_mode_select.GetSelection() == 1 and
@@ -1726,7 +1736,10 @@ class LUTFrame(BaseFrame):
 		if self.worker.tempdir and os.path.isdir(self.worker.tempdir):
 			self.worker.wrapup(False)
 		config.writecfg(module="curve-viewer", options=("display.number", ))
-		event.Skip()
+		# Hide first (looks nicer)
+		self.Hide()
+		# Need to use CallAfter to prevent hang under Windows if minimized
+		wx.CallAfter(self.Destroy)
 
 	def OnMotion(self, event):
 		if isinstance(event, wx.MouseEvent):
@@ -1930,7 +1943,7 @@ class LUTFrame(BaseFrame):
 													 (("R", R), ("G", G), ("B", B)))])
 						# Note: We need to make sure each point is a float because it
 						# might be a decimal.Decimal, which can't be divided by floats!
-						gamma.append(label + " %.2f" % (math.log(float(y) / axis_y) / math.log(x / 255.0)))
+						gamma.append(label + " %.2f" % round(math.log(float(y) / axis_y) / math.log(x / 255.0), 2))
 						if "=" in label:
 							break
 					if gamma:
