@@ -859,7 +859,6 @@ class GamutViewOptions(wx_Panel):
 							   flag=wx.ALIGN_CENTER_VERTICAL | wx.EXPAND)
 		self.direction_select.Bind(wx.EVT_CHOICE, self.direction_select_handler)
 		self.direction_select.SetSelection(0)
-		self.direction_select.Hide()
 
 		self.sizer.Add((0, 0))
 
@@ -991,7 +990,7 @@ class GamutViewOptions(wx_Panel):
 	def toggle_clut_handler(self, event):
 		parent = self.TopLevelParent
 		self.Freeze()
-		self.direction_select.Show(bool(parent.profile) and 
+		self.direction_select.Enable(bool(parent.profile) and 
 								   "B2A0" in parent.profile.tags and
 								   "A2B0" in parent.profile.tags and
 								   self.toggle_clut.GetValue())
@@ -1043,10 +1042,9 @@ class ProfileInfoFrame(LUTFrame):
 		self.plot_mode_sizer.Add(self.plot_mode_select,
 								 flag=wx.ALIGN_CENTER_VERTICAL)
 		self.plot_mode_select.Bind(wx.EVT_CHOICE, self.plot_mode_select_handler)
-		self.plot_mode_select.SetSelection(2)
 		self.plot_mode_select.Disable()
 		
-		self.tooltip_btn = BitmapButton(p1, -1, geticon(16, "dialog-information"),
+		self.tooltip_btn = BitmapButton(p1, -1, geticon(16, "question-inverted"),
 										style=wx.NO_BORDER)
 		self.tooltip_btn.SetBackgroundColour(BGCOLOUR)
 		self.tooltip_btn.Bind(wx.EVT_BUTTON, self.tooltip_handler)
@@ -1055,7 +1053,7 @@ class ProfileInfoFrame(LUTFrame):
 														wx.LEFT, border=8)
 
 		self.save_plot_btn = BitmapButton(p1, -1,
-										  geticon(16, "camera-photo"),
+										  geticon(16, "image-x-generic-inverted"),
 										  style=wx.NO_BORDER)
 		self.save_plot_btn.SetBackgroundColour(BGCOLOUR)
 		self.save_plot_btn.Bind(wx.EVT_BUTTON, self.SaveFile)
@@ -1334,10 +1332,12 @@ class ProfileInfoFrame(LUTFrame):
 													profile.tags.get("kTRC"))
 		self.trc = None
 		
-		self.gamut_view_options.direction_select.Show("B2A0" in
-													  self.profile.tags and
-													  "A2B0" in
-													  self.profile.tags)
+		self.gamut_view_options.direction_select.Enable("B2A0" in
+													    self.profile.tags and
+													    "A2B0" in
+													    self.profile.tags)
+		if not self.gamut_view_options.direction_select.Enabled:
+			self.gamut_view_options.direction_select.SetSelection(0)
 		self.gamut_view_options.toggle_clut.SetValue("B2A0" in profile.tags or
 													 "A2B0" in profile.tags)
 		self.gamut_view_options.toggle_clut.Show("B2A0" in profile.tags or
@@ -1367,10 +1367,29 @@ class ProfileInfoFrame(LUTFrame):
 				wx.CallAfter(show_result_dialog, exception, self)
 			else:
 				choice.append(lang.getstr("[rgb]TRC"))
+		if getcfg("show_advanced_options"):
+			if isinstance(self.profile.tags.get("A2B0"), ICCP.LUT16Type):
+				choice.append(lang.getstr('profile.tags.A2B0.shaper_curves.input'))
+				choice.append(lang.getstr('profile.tags.A2B0.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("A2B1"), ICCP.LUT16Type):
+				choice.append(lang.getstr('profile.tags.A2B1.shaper_curves.input'))
+				choice.append(lang.getstr('profile.tags.A2B1.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("A2B2"), ICCP.LUT16Type):
+				choice.append(lang.getstr('profile.tags.A2B2.shaper_curves.input'))
+				choice.append(lang.getstr('profile.tags.A2B2.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A0"), ICCP.LUT16Type):
+				choice.append(lang.getstr('profile.tags.B2A0.shaper_curves.input'))
+				choice.append(lang.getstr('profile.tags.B2A0.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A1"), ICCP.LUT16Type):
+				choice.append(lang.getstr('profile.tags.B2A1.shaper_curves.input'))
+				choice.append(lang.getstr('profile.tags.B2A1.shaper_curves.output'))
+			if isinstance(self.profile.tags.get("B2A2"), ICCP.LUT16Type):
+				choice.append(lang.getstr('profile.tags.B2A2.shaper_curves.input'))
+				choice.append(lang.getstr('profile.tags.B2A2.shaper_curves.output'))
 		choice.append(lang.getstr("gamut"))
 		self.Freeze()
 		self.plot_mode_select.SetItems(choice)
-		if plot_mode_count != self.plot_mode_select.GetCount():
+		if plot_mode < 0:
 			plot_mode = self.plot_mode_select.GetCount() - 1
 		self.plot_mode_select.SetSelection(min(plot_mode,
 											   self.plot_mode_select.GetCount() - 1))
@@ -1482,7 +1501,10 @@ class ProfileInfoFrame(LUTFrame):
 												 "last_icc_path",
 												 "position.profile_info",
 												 "size.profile_info"))
-		event.Skip()
+		# Hide first (looks nicer)
+		self.Hide()
+		# Need to use CallAfter to prevent hang under Windows if minimized
+		wx.CallAfter(self.Destroy)
 
 	def OnMotion(self, event):
 		if isinstance(event, wx.MouseEvent):
