@@ -5,12 +5,16 @@ import os
 import sys
 
 from ICCProfile import ICCProfile
-from config import get_data_path, get_verified_path, getcfg, hascfg, setcfg
-from log import safe_print
+from config import (enc, get_data_path, get_verified_path, getcfg, hascfg,
+					profile_ext, setcfg)
+from log import log, safe_print
 from meta import name as appname
 from options import debug
+from ordereddict import OrderedDict
+from util_io import Files
 from util_os import waccess
-from worker import Error, show_result_dialog
+from util_str import safe_str
+from worker import Error, FilteredStream, LineBufferedStream, show_result_dialog
 import ICCProfile as ICCP
 import colormath
 import config
@@ -18,6 +22,7 @@ import localization as lang
 import worker
 from wxwindows import BaseApp, BaseFrame, FileDrop, InfoDialog, wx
 from wxfixes import TempXmlResource
+from wxLUT3DFrame import LUT3DFrame
 import floatspin
 import xh_floatspin
 import xh_bitmapctrls
@@ -27,7 +32,12 @@ from wx import xrc
 
 class SynthICCFrame(BaseFrame):
 
-	""" 3D LUT creation window """
+	""" Synthetic ICC creation window """
+
+	# Shared methods from 3D LUT UI
+	for lut3d_ivar_name, lut3d_ivar in LUT3DFrame.__dict__.iteritems():
+		if lut3d_ivar_name.startswith("lut3d_"):
+			locals()[lut3d_ivar_name] = lut3d_ivar
 	
 	def __init__(self, parent=None):
 		self.res = TempXmlResource(get_data_path(os.path.join("xrc", 
@@ -86,6 +96,7 @@ class SynthICCFrame(BaseFrame):
 		self.trc_gamma_ctrl.Bind(wx.EVT_COMBOBOX, self.trc_gamma_ctrl_handler)
 		self.trc_gamma_ctrl.Bind(wx.EVT_KILL_FOCUS, self.trc_gamma_ctrl_handler)
 		self.trc_gamma_type_ctrl.Bind(wx.EVT_CHOICE, self.trc_gamma_type_ctrl_handler)
+		self.lut3d_bind_hdr_trc_handlers()
 		self.black_output_offset_ctrl.Bind(wx.EVT_SLIDER,
 										   self.black_output_offset_ctrl_handler)
 		self.black_output_offset_intctrl.Bind(wx.EVT_TEXT,
@@ -142,9 +153,13 @@ class SynthICCFrame(BaseFrame):
 			config.writecfg(module="synthprofile",
 							options=("synthprofile.", "last_icc_path",
 									 "position.synthiccframe",
-									 "size.synthiccframe"))
+									 "size.synthiccframe",
+									 "3dlut.hdr_"))
 		if event:
-			event.Skip()
+			# Hide first (looks nicer)
+			self.Hide()
+			# Need to use CallAfter to prevent hang under Windows if minimized
+			wx.CallAfter(self.Destroy)
 	
 	def black_luminance_ctrl_handler(self, event):
 		v = self.black_luminance_ctrl.GetValue()
@@ -182,11 +197,11 @@ class SynthICCFrame(BaseFrame):
 		if (v != old and (old == 0 or v == 0)) or event is True:
 			self.Freeze()
 			i = self.trc_ctrl.GetSelection()
-			self.trc_gamma_type_ctrl.Show(i in (0, 4) and bool(v))
+			self.trc_gamma_type_ctrl.Show(i in (0, 5) and bool(v))
 			if not v:
 				self.bpc_ctrl.SetValue(False)
 			self.bpc_ctrl.Enable(bool(v))
-			black_output_offset_show = (i in (0, 4) and bool(v))
+			black_output_offset_show = (i in (0, 5, 7, 8) and bool(v))
 			self.black_output_offset_label.Show(black_output_offset_show)
 			self.black_output_offset_ctrl.Show(black_output_offset_show)
 			self.black_output_offset_intctrl.Show(black_output_offset_show)
@@ -310,7 +325,7 @@ class SynthICCFrame(BaseFrame):
 				trc = ICCP.CurveType(profile=profile)
 				for XYZ in colors[5:]:
 					trc.append(XYZ[1] / colors[0][1] * 65535)
-				transfer_function = trc.get_transfer_function()
+				transfer_function = trc.get_transfer_function(outoffset=1.0)
 				if transfer_function and transfer_function[1] >= .95:
 					# Use detected transfer function
 					gamma = transfer_function[0][1]
@@ -355,6 +370,7 @@ class SynthICCFrame(BaseFrame):
 	def luminance_ctrl_handler(self, event):
 		v = self.luminance_ctrl.GetValue()
 		setcfg("synthprofile.luminance", v)
+		self.lut3d_set_option("3dlut.hdr_peak_luminance", v)
 		self.black_luminance_ctrl_handler(event)
 	
 	def parse_XYZ(self, name, set_blackpoint=False):
@@ -438,6 +454,12 @@ class SynthICCFrame(BaseFrame):
 			gamma, white, red, green, blue = colormath.rgb_spaces[preset_name]
 			white = colormath.get_whitepoint(white)
 			self._updating_ctrls = True
+			self.panel.Freeze()
+			if self.preset_ctrl.GetStringSelection() == "DCI P3":
+				tech = self.tech["dcpj"]
+			else:
+				tech = self.tech[""]
+			self.tech_ctrl.SetStringSelection(tech)
 			for i, component in enumerate("XYZ"):
 				getattr(self, "white_%s" % component).SetValue(white[i] * 100)
 			self.parse_XYZ("white", True)
@@ -446,6 +468,7 @@ class SynthICCFrame(BaseFrame):
 					getattr(self, "%s_%s" % (color, component)).SetValue(locals()[color][i])
 			self.parse_xy(None)
 			self.set_trc(gamma)
+			self.panel.Thaw()
 			self._updating_ctrls = False
 
 	def get_commands(self):
@@ -473,25 +496,28 @@ class SynthICCFrame(BaseFrame):
 		if gamma == -1023:
 			# DICOM
 			self.trc_ctrl.SetSelection(1)
+		elif gamma == -2.0:
+			# HLG
+			self.trc_ctrl.SetSelection(2)
 		elif gamma == -3.0:
 			# L*
-			self.trc_ctrl.SetSelection(2)
+			self.trc_ctrl.SetSelection(3)
 		elif gamma == -709:
 			# Rec. 709
-			self.trc_ctrl.SetSelection(3)
+			self.trc_ctrl.SetSelection(4)
 		elif gamma == -1886:
 			# Rec. 1886
-			self.trc_ctrl.SetSelection(4)
+			self.trc_ctrl.SetSelection(5)
 			self.trc_ctrl_handler()
 		elif gamma == -240:
 			# SMPTE 240M
-			self.trc_ctrl.SetSelection(5)
-		elif gamma == -2084:
-			# SMPTE 2084
 			self.trc_ctrl.SetSelection(6)
+		elif gamma == -2084:
+			# SMPTE 2084, roll-off clip
+			self.trc_ctrl.SetSelection(8)
 		elif gamma == -2.4:
 			# sRGB
-			self.trc_ctrl.SetSelection(7)
+			self.trc_ctrl.SetSelection(9)
 		else:
 			# Gamma
 			self.trc_ctrl.SetSelection(0)
@@ -508,42 +534,102 @@ class SynthICCFrame(BaseFrame):
 		self.parse_xy("red")
 	
 	def save_as_btn_handler(self, event):
-		XYZ = self.get_XYZ()
 		try:
 			gamma = float(self.trc_gamma_ctrl.Value)
 		except ValueError:
 			wx.Bell()
 			gamma = 2.2
 			self.trc_gamma_ctrl.Value = str(gamma)
-		white = XYZ["wX"], XYZ["wY"], XYZ["wZ"]
-		if self.trc_ctrl.GetSelection() in (0, 1, 4, 6):
-			# 0 = Gamma
-			# 1 = DICOM - trc set here is not actually used in the end
-			# 4 = Rec. 1886 - trc set here is only used if black = 0
-			# 6 = SMPTE 2084 - trc set here is not actually used in the end
-			trc = gamma
-		elif self.trc_ctrl.GetSelection() == 2:
-			# L*
-			trc = -3.0
-		elif self.trc_ctrl.GetSelection() == 3:
-			# Rec. 709
-			trc = -709
-		elif self.trc_ctrl.GetSelection() == 5:
-			# SMPTE 240M
-			trc = -240
-		elif self.trc_ctrl.GetSelection() == 7:
-			# sRGB
-			trc = -2.4
+
 		defaultDir, defaultFile = get_verified_path("last_icc_path")
 		defaultFile = lang.getstr("unnamed")
-		if self.colorspace_rgb_ctrl.Value:
+		path = None
+		dlg = wx.FileDialog(self, 
+							lang.getstr("save_as"),
+							defaultDir=defaultDir,
+							defaultFile=defaultFile,
+							wildcard=lang.getstr("filetype.icc") + 
+									 "|*" + profile_ext, 
+							style=wx.SAVE | wx.FD_OVERWRITE_PROMPT)
+		dlg.Center(wx.BOTH)
+		if dlg.ShowModal() == wx.ID_OK:
+			path = dlg.GetPath()
+		dlg.Destroy()
+		if path:
+			if os.path.splitext(path)[1].lower() not in (".icc", ".icm"):
+				path += profile_ext
+			if not waccess(path, os.W_OK):
+				show_result_dialog(Error(lang.getstr("error.access_denied.write",
+													 path)),
+								   self)
+				return
+			setcfg("last_icc_path", path)
+		else:
+			return
+
+		XYZ = self.get_XYZ()
+		if self.trc_ctrl.GetSelection() in (0, 5):
+			# 0 = Gamma
+			# 5 = Rec. 1886 - trc set here is only used if black = 0
+			trc = gamma
+		elif self.trc_ctrl.GetSelection() == 1:
+			# DICOM
+			trc = -1
+		elif self.trc_ctrl.GetSelection() == 2:
+			# HLG
+			trc = -2
+		elif self.trc_ctrl.GetSelection() == 3:
+			# L*
+			trc = -3.0
+		elif self.trc_ctrl.GetSelection() == 4:
+			# Rec. 709
+			trc = -709
+		elif self.trc_ctrl.GetSelection() == 6:
+			# SMPTE 240M
+			trc = -240
+		elif self.trc_ctrl.GetSelection() in (7, 8):
+			# SMPTE 2084
+			trc = -2084
+		elif self.trc_ctrl.GetSelection() == 9:
+			# sRGB
+			trc = -2.4
+		rolloff = self.trc_ctrl.GetSelection() == 8
+		class_i = self.profile_class_ctrl.GetSelection()
+		tech_i = self.tech_ctrl.GetSelection()
+		ciis_i = self.ciis_ctrl.GetSelection()
+		consumer = lambda result: (isinstance(result, Exception) and
+								   show_result_dialog(result, self))
+		wargs = (XYZ, trc, path)
+		wkwargs = {"rgb": self.colorspace_rgb_ctrl.Value,
+				   "rolloff": rolloff,
+				   "bpc": self.bpc_ctrl.Value,
+				   "profile_class": self.profile_classes.keys()[class_i],
+				   "tech": self.tech.keys()[tech_i],
+				   "ciis": self.ciis.keys()[ciis_i]}
+		if (trc == -2084 and rolloff) or trc == -2:
+			if trc == -2084:
+				msg = "smpte2084.rolloffclip"
+			else:
+				msg = "hlg"
+			self.worker.recent.write(lang.getstr("trc." + msg) + "\n")
+			self.worker.start(consumer,
+							  self.create_profile, wargs=(XYZ, trc, path),
+							  wkwargs=wkwargs,
+							  progress_msg=lang.getstr("synthicc.create"))
+		else:
+			consumer(self.create_profile(*wargs, **wkwargs))
+
+	def create_profile(self, XYZ, trc, path, rgb=True, rolloff=True, bpc=False,
+					   profile_class="mntr", tech=None, ciis=None):
+		white = XYZ["wX"], XYZ["wY"], XYZ["wZ"]
+		if rgb:
 			# Color profile
 			profile = ICCP.ICCProfile.from_XYZ((XYZ["rX"], XYZ["rY"], XYZ["rZ"]),
 											   (XYZ["gX"], XYZ["gY"], XYZ["gZ"]),
 											   (XYZ["bX"], XYZ["bY"], XYZ["bZ"]),
 											   (XYZ["wX"], XYZ["wY"], XYZ["wZ"]),
-											   trc,
-											   defaultFile,
+											   1.0,
+											   "",
 											   getcfg("copyright"))
 			black = colormath.adapt(XYZ["kX"], XYZ["kY"], XYZ["kZ"], white)
 			profile.tags.rTRC = ICCP.CurveType(profile=profile)
@@ -563,11 +649,16 @@ class SynthICCFrame(BaseFrame):
 								  getcfg("synthprofile.luminance"))] * 3
 			profile.tags.kTRC = ICCP.CurveType(profile=profile)
 			channels = "k"
-		if self.trc_ctrl.GetSelection() == 1:
+		if trc == -2:
+			# HLG
+			outoffset = 1
+		else:
+			outoffset = getcfg("synthprofile.trc_output_offset")
+		if trc == -1:
 			# DICOM
 			# Absolute luminance values!
 			try:
-				if self.colorspace_rgb_ctrl.Value:
+				if rgb:
 					# Color profile
 					profile.set_dicom_trc([v * getcfg("synthprofile.luminance")
 										   for v in black],
@@ -577,54 +668,113 @@ class SynthICCFrame(BaseFrame):
 					profile.tags.kTRC.set_dicom_trc(getcfg("synthprofile.black_luminance"),
 													getcfg("synthprofile.luminance"))
 			except ValueError, exception:
-				show_result_dialog(exception, self)
-				return
-		elif self.trc_ctrl.GetSelection() in (0, 4) and black != [0, 0, 0]:
+				return exception
+		elif trc > -1 and black != [0, 0, 0]:
 			# Gamma with output offset or Rec. 1886-like
-			outoffset = getcfg("synthprofile.trc_output_offset")
-			if self.colorspace_rgb_ctrl.Value:
+			if rgb:
 				# Color profile
-				profile.set_bt1886_trc(black, outoffset, gamma,
+				profile.set_bt1886_trc(black, outoffset, trc,
 									   getcfg("synthprofile.trc_gamma_type"))
 			else:
 				# Grayscale profile
-				profile.tags.kTRC.set_bt1886_trc(black[1], outoffset, gamma,
+				profile.tags.kTRC.set_bt1886_trc(black[1], outoffset, trc,
 												 getcfg("synthprofile.trc_gamma_type"))
-		elif self.trc_ctrl.GetSelection() == 6:
-			# SMPTE 2084
-			if self.colorspace_rgb_ctrl.Value:
+		elif trc == -2084 or trc == -2:
+			# SMPTE 2084 or HLG
+			if trc == -2084:
+				hdr_format = "PQ"
+			else:
+				hdr_format = "HLG"
+			minmll = getcfg("3dlut.hdr_minmll")
+			if rolloff:
+				maxmll = getcfg("3dlut.hdr_maxmll")
+			else:
+				maxmll = getcfg("synthprofile.luminance")
+			if rgb:
 				# Color profile
-				profile.set_smpte2084_trc([v * getcfg("synthprofile.luminance")
-										   for v in black],
-										  getcfg("synthprofile.luminance"))
+				if trc == -2084:
+					profile.set_smpte2084_trc([v * getcfg("synthprofile.luminance") *
+											   (1 - outoffset)
+											   for v in black],
+											  getcfg("synthprofile.luminance"),
+											  minmll, maxmll,
+											  rolloff=True,
+											  blend_blackpoint=False)
+				else:
+					# HLG
+					profile.set_hlg_trc((0, 0, 0),
+										getcfg("synthprofile.luminance"),
+										1.2,
+										getcfg("3dlut.hdr_ambient_luminance"),
+										blend_blackpoint=False)
+				if rolloff or trc == -2:
+					rgb_space = profile.get_rgb_space()
+					rgb_space[0] = 1.0  # Set gamma to 1.0 (not actually used)
+					rgb_space = colormath.get_rgb_space(rgb_space)
+					linebuffered_logfiles = []
+					if sys.stdout.isatty():
+						linebuffered_logfiles.append(safe_print)
+					else:
+						linebuffered_logfiles.append(log)
+					logfiles = Files([LineBufferedStream(
+										FilteredStream(Files(linebuffered_logfiles),
+													   enc, discard="",
+													   linesep_in="\n", 
+													   triggers=[])),
+									  self.worker.recent,
+									  self.worker.lastmsg])
+					quality = getcfg("profile.quality")
+					clutres = {"m": 17, "l": 9}.get(quality, 33)
+					profile.tags.A2B0 = ICCP.create_synthetic_hdr_clut_profile(
+						hdr_format,
+						rgb_space, "",
+						getcfg("synthprofile.black_luminance") * (1 - outoffset),
+						getcfg("synthprofile.luminance"), minmll, maxmll,
+						1.2, getcfg("3dlut.hdr_ambient_luminance"),
+						clutres=clutres, worker=self.worker,
+						logfile=logfiles).tags.A2B0
+				if black != [0, 0, 0] and outoffset and not bpc:
+					profile.apply_black_offset(black)
+				if rolloff or trc == -2:
+					profile.write(path)
+					self.worker.update_profile_B2A(profile, not bpc,
+												   smooth=False,
+												   rgb_space=rgb_space)
 			else:
 				# Grayscale profile
-				profile.tags.kTRC.set_smpte2084_trc(getcfg("synthprofile.black_luminance"),
-													getcfg("synthprofile.luminance"))
+				if trc == -2084:
+					profile.tags.kTRC.set_smpte2084_trc(getcfg("synthprofile.black_luminance") *
+														(1 - outoffset),
+														getcfg("synthprofile.luminance"),
+														minmll, maxmll,
+														rolloff=True)
+				else:
+					# HLG
+					profile.tags.kTRC.set_smpte2084_trc(0,
+														1.2,
+														getcfg("3dlut.hdr_ambient_luminance"))
+				if black != [0, 0, 0] and outoffset and not bpc:
+					profile.tags.kTRC.apply_bpc(black[1])
 		elif black != [0, 0, 0]:
-			if self.colorspace_rgb_ctrl.Value:
+			if rgb:
 				# Color profile
-				rXYZ = profile.tags.rXYZ.values()
-				gXYZ = profile.tags.gXYZ.values()
-				bXYZ = profile.tags.bXYZ.values()
-				mtx = colormath.Matrix3x3([[rXYZ[0], gXYZ[0], bXYZ[0]],
-										   [rXYZ[1], gXYZ[1], bXYZ[1]],
-										   [rXYZ[2], gXYZ[2], bXYZ[2]]])
-				rgbblack = mtx.inverted() * black
+				vmin = 0
 			else:
 				# Grayscale profile
-				rgbblack = black
-			# Optimize for uInt16Number encoding
-			rgbblack = [round(max(v, 0) * 65535) / 65535 for v in rgbblack]
+				vmin = black[1]
 			for i, channel in enumerate(channels):
 				TRC = profile.tags["%sTRC" % channel]
-				TRC.set_trc(trc, 1024, vmin=rgbblack[i] * 65535)
+				TRC.set_trc(trc, 1024, vmin=vmin * 65535)
+			if rgb:
+				profile.apply_black_offset(black)
 		else:
 			for channel in channels:
 				profile.tags["%sTRC" % channel].set_trc(trc, 1)
-		if black != [0, 0, 0] and self.bpc_ctrl.Value:
-			for channel in channels:
-				profile.tags["%sTRC" % channel].apply_bpc()
+		if black != [0, 0, 0] and bpc:
+			if rgb:
+				profile.apply_black_offset((0, 0, 0))
+			else:
+				profile.tags.kTRC.apply_bpc()
 		for tagname in ("lumi", "bkpt"):
 			if tagname == "lumi":
 				# Absolute
@@ -636,32 +786,24 @@ class SynthICCFrame(BaseFrame):
 			(profile.tags[tagname].X,
 			 profile.tags[tagname].Y,
 			 profile.tags[tagname].Z) = X, Y, Z
-		path = None
-		dlg = wx.FileDialog(self, 
-							lang.getstr("save_as"),
-							defaultDir=defaultDir,
-							defaultFile=defaultFile,
-							wildcard="*.icc", 
-							style=wx.SAVE | wx.FD_OVERWRITE_PROMPT)
-		dlg.Center(wx.BOTH)
-		if dlg.ShowModal() == wx.ID_OK:
-			path = dlg.GetPath()
-		dlg.Destroy()
-		if path:
-			if os.path.splitext(path)[1].lower() not in (".icc", ".icm"):
-				path += ".icc"
-			if not waccess(path, os.W_OK):
-				show_result_dialog(Error(lang.getstr("error.access_denied.write",
-													 path)),
-								   self)
-				return
-			setcfg("last_icc_path", path)
-			profile.setDescription(os.path.splitext(os.path.basename(path))[0])
-			profile.calculateID()
-			try:
-				profile.write(path)
-			except Exception, exception:
-				show_result_dialog(exception, self)
+		# Profile class
+		profile.profileClass = profile_class
+		# Technology type
+		if tech:
+			profile.tags.tech = ICCP.SignatureType("sig \0\0\0\0" +
+												   tech,
+												   "tech")
+		# Colorimetric intent image state
+		if ciis:
+			profile.tags.ciis = ICCP.SignatureType("sig \0\0\0\0" +
+												   ciis,
+												   "ciis")
+		profile.setDescription(os.path.splitext(os.path.basename(path))[0])
+		profile.calculateID()
+		try:
+			profile.write(path)
+		except Exception, exception:
+			return exception
 	
 	def setup_language(self):
 		BaseFrame.setup_language(self)
@@ -676,12 +818,31 @@ class SynthICCFrame(BaseFrame):
 		self.trc_gamma_types_ba = {"g": 0, "G": 1}
 		self.trc_gamma_type_ctrl.SetItems([lang.getstr("trc.type.relative"),
 										   lang.getstr("trc.type.absolute")])
+
+		self.profile_classes = OrderedDict(get_mapping(ICCP.profileclass.items(),
+													   ["mntr", "scnr"]))
+		self.profile_class_ctrl.SetItems(self.profile_classes.values())
+		self.profile_class_ctrl.SetSelection(0)
+
+		self.tech = OrderedDict(get_mapping([("", "unspecified")] +
+											ICCP.tech.items(),
+											["", "fscn", "dcam", "rscn", "vidm",
+											 "vidc", "pjtv", "CRT ", "PMD ",
+											 "AMD ", "mpfs", "dmpc", "dcpj"]))
+		self.tech_ctrl.SetItems(self.tech.values())
+		self.tech_ctrl.SetSelection(0)
+
+		self.ciis = OrderedDict(get_mapping([("", "unspecified")] +
+											 ICCP.ciis.items(),
+											["", "scoe", "sape", "fpce"]))
+		self.ciis_ctrl.SetItems(self.ciis.values())
+		self.ciis_ctrl.SetSelection(0)
 	
 	def trc_ctrl_handler(self, event=None):
 		if not self._updating_ctrls:
 			self.preset_ctrl.SetSelection(0)
 		i = self.trc_ctrl.GetSelection()
-		if i == 4:
+		if i == 5:
 			# BT.1886
 			setcfg("synthprofile.trc_gamma", 2.4)
 			setcfg("synthprofile.trc_gamma_type", "G")
@@ -721,30 +882,62 @@ class SynthICCFrame(BaseFrame):
 		self.update_trc_controls()
 
 	def update_trc_control(self):
-		if self.trc_ctrl.GetSelection() in (0, 4):
+		if self.trc_ctrl.GetSelection() in (0, 5):
 			if (getcfg("synthprofile.trc_gamma_type") == "G" and
 				getcfg("synthprofile.trc_output_offset") == 0 and
 				getcfg("synthprofile.trc_gamma") == 2.4):
-				self.trc_ctrl.SetSelection(4)  # BT.1886
+				self.trc_ctrl.SetSelection(5)  # BT.1886
 			else:
 				self.trc_ctrl.SetSelection(0)  # Gamma
 
 	def update_trc_controls(self):
 		i = self.trc_ctrl.GetSelection()
 		self.panel.Freeze()
-		self.trc_gamma_label.Show(i in (0, 4))
+		self.trc_gamma_label.Show(i in (0, 5))
 		self.trc_gamma_ctrl.SetValue(str(getcfg("synthprofile.trc_gamma")))
-		self.trc_gamma_ctrl.Show(i in (0, 4))
+		self.trc_gamma_ctrl.Show(i in (0, 5))
 		self.trc_gamma_type_ctrl.SetSelection(self.trc_gamma_types_ba[getcfg("synthprofile.trc_gamma_type")])
 		self.panel.GetSizer().Layout()
 		self.panel.Thaw()
-		if i in (0, 4):
+		if i in (0, 5, 7, 8):
+			# Gamma, BT.1886, SMPTE 2084 (PQ)
 			outoffset = int(getcfg("synthprofile.trc_output_offset") * 100)
 		else:
 			outoffset = 100
 		self.black_output_offset_ctrl.SetValue(outoffset)
 		self.black_output_offset_intctrl.SetValue(outoffset)
+		self.lut3d_hdr_minmll_ctrl.SetValue(getcfg("3dlut.hdr_minmll"))
+		self.lut3d_hdr_maxmll_ctrl.SetValue(getcfg("3dlut.hdr_maxmll"))
+		setcfg("3dlut.hdr_peak_luminance", getcfg("synthprofile.luminance"))
+		self.lut3d_hdr_update_diffuse_white()
+		self.lut3d_hdr_ambient_luminance_ctrl.SetValue(getcfg("3dlut.hdr_ambient_luminance"))
+		self.lut3d_hdr_update_system_gamma()
+		self.lut3d_hdr_minmll_label.Show(i in (7, 8))  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_minmll_ctrl.Show(i in (7, 8))  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_minmll_ctrl_label.Show(i in (7, 8))  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_maxmll_label.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_maxmll_ctrl.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_maxmll_ctrl_label.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_diffuse_white_label.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_diffuse_white_txt.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_diffuse_white_txt_label.Show(i == 8)  # SMPTE 2084 (PQ)
+		self.lut3d_hdr_ambient_luminance_label.Show(i == 2)  # HLG
+		self.lut3d_hdr_ambient_luminance_ctrl.Show(i == 2)  # HLG
+		self.lut3d_hdr_ambient_luminance_ctrl_label.Show(i == 2)  # HLG
+		self.lut3d_hdr_system_gamma_label.Show(i == 2)  # HLG
+		self.lut3d_hdr_system_gamma_txt.Show(i == 2)  # HLG
 		self.black_luminance_ctrl_handler(True)
+		if i in (4, 6):
+			# Rec 709 or SMPTE 240M
+			# Match Adobe 'video' profiles
+			self.profile_class_ctrl.SetStringSelection(self.profile_classes["scnr"])
+			self.tech_ctrl.SetStringSelection(self.tech["vidc"])
+			self.ciis_ctrl.SetStringSelection(self.ciis["fpce"])
+		elif self.profile_class_ctrl.GetStringSelection() == self.profile_classes["scnr"]:
+			# If 'input' profile, reset class/tech/colorimetric intent image state
+			self.profile_class_ctrl.SetStringSelection(self.profile_classes["mntr"])
+			self.tech_ctrl.SetStringSelection(self.tech[""])
+			self.ciis_ctrl.SetStringSelection(self.ciis[""])
 	
 	def white_XYZ_ctrl_handler(self, event):
 		self.parse_XYZ("white")
@@ -755,6 +948,12 @@ class SynthICCFrame(BaseFrame):
 		
 
 
+def get_mapping(mapping, keys):
+	return sorted([(k, lang.getstr(v.lower().replace(" ", "_"))) for k, v in
+				   filter(lambda item: item[0] in keys, mapping)],
+				  key=lambda item: item[0])
+
+
 def main():
 	config.initcfg("synthprofile")
 	lang.init()
