@@ -1,3 +1,5 @@
+# -*- coding: utf-8 -*-
+
 """
 Visual whitepoint editor
 
@@ -23,8 +25,9 @@ from wxfixes import wx
 from wx.lib.agw import aui
 from wx.lib.intctrl import IntCtrl
 
-from config import (defaults, fs_enc, getbitmap, getcfg, get_argyll_display_number,
-                    get_default_dpi, get_icon_bundle, geticon, initcfg,
+from config import (defaults, fs_enc, getbitmap, getcfg,
+                    get_argyll_display_number, get_default_dpi,
+                    get_display_name, get_icon_bundle, geticon, initcfg,
                     profile_ext, setcfg)
 from log import safe_print
 from meta import name as appname
@@ -82,6 +85,21 @@ def s(i):
     return i * max(getcfg("app.dpi") / get_default_dpi(), 1)
 
 
+def update_patterngenerator(self):
+    while self and wx.App.IsMainLoopRunning():
+        if self.update_patterngenerator_event.wait(0.05):
+            self.update_patterngenerator_event.clear()
+            x, y, size = self.patterngenerator_config
+            self.patterngenerator.send((self._colour.r / 255.0,
+                                        self._colour.g / 255.0,
+                                        self._colour.b / 255.0),
+                                       (self._bgcolour.r / 255.0,
+                                        self._bgcolour.g / 255.0,
+                                        self._bgcolour.b / 255.0),
+                                       x=x, y=y, w=size, h=size)
+        sleep(0.05)
+
+
 def _show_result_after(*args, **kwargs):
     wx.CallAfter(show_result_dialog, *args, **kwargs)
 
@@ -1417,6 +1435,8 @@ class HSlider(BaseLineCtrl):
     def GetMax(self):
         return self.maxval
 
+    def GetMin(self):
+        return self.minval
 
     def SetMax(self, maxval):
         self.maxval = maxval
@@ -1604,7 +1624,7 @@ class ProfileManager(object):
 
     managers = []
 
-    def __init__(self, window):
+    def __init__(self, window, geometry=None, profile=None):
         self._display = window.GetDisplay()
         self._lock = threading.Lock()
         self._profiles = {}
@@ -1618,32 +1638,59 @@ class ProfileManager(object):
         self._window.Bind(wx.EVT_DISPLAY_CHANGED, self.display_changed_handler)
         self._worker = Worker()
         ProfileManager.managers.append(self)
-        self.update()
+        self.update(False)
+        self._profiles_overridden = {}
+        if geometry and profile:
+            self._profiles_overridden[geometry] = profile
 
 
     def _manage_display(self, display_no, geometry):
         # Has to be thread-safe!
         with self._lock:
             try:
-                profile = ICCP.get_display_profile(display_no)
+                display_profile = ICCP.get_display_profile(display_no)
             except (ICCP.ICCProfileInvalidError, IOError,
                     IndexError), exception:
                 safe_print("Could not get display profile for display %i" %
                            (display_no + 1), "@ %i, %i, %ix%i:" %
                            geometry, exception)
             else:
-                if profile and profile.ID != self._srgb_profile.ID:
+                profile = self._profiles_overridden.get(geometry)
+                if not profile:
+                    profile = display_profile
+                if display_profile and display_profile.ID != self._srgb_profile.ID:
+                    # Set initial whitepoint according to vcgt
+                    # Convert calibration information from embedded WCS
+                    # profile (if present) to VideCardGammaType if the
+                    # latter is not present
+                    if (isinstance(profile.tags.get("MS00"),
+                                   ICCP.WcsProfilesTagType) and
+                        not "vcgt" in profile.tags):
+                        profile.tags["vcgt"] = profile.tags["MS00"].get_vcgt()
+                    if isinstance(profile.tags.get("vcgt"),
+                                  ICCP.VideoCardGammaType):
+                        values = profile.tags.vcgt.getNormalizedValues()
+                        RGB = []
+                        for i in xrange(3):
+                            RGB.append(int(round(values[-1][i] * 255)))
+                        (self._window._colour.r,
+                         self._window._colour.g,
+                         self._window._colour.b) = RGB
+                        self._window._colour.ToHSV()
+                        wx.CallAfter(self._window.DrawAll)
                     # Remember profile, but discard profile filename
                     # (Important - can't re-install profile from same path
                     # where it is installed!)
-                    if not self._set_profile_temp_filename(profile):
+                    if not self._set_profile_temp_filename(display_profile):
                         return
-                    self._profiles[geometry] = profile
+                    self._profiles[geometry] = display_profile
                     self._install_profile(display_no, self._srgb_profile)
 
 
     def _install_profile(self, display_no, profile, wrapup=False):
         # Has to be thread-safe!
+        if self._window.patterngenerator:
+            return
         dispwin = get_argyll_util("dispwin")
         if not dispwin:
             _show_result_after(Error(lang.getstr("argyll.util.not_found",
@@ -1694,17 +1741,25 @@ class ProfileManager(object):
             self._update_timer.Stop()
     
     
-    def update(self):
+    def update(self, restore_display_profiles=True):
         """
         Clear calibration on the current display, and restore it on
         the previous one (if any)
         
         """
-        self.restore_display_profiles()
+        if restore_display_profiles:
+            self.restore_display_profiles()
         display_no = get_argyll_display_number(self._display.Geometry)
         if display_no is not None:
             threading.Thread(target=self._manage_display,
                              args=(display_no, self._display.Geometry.Get())).start()
+            if not self._window.patterngenerator:
+                display_name = get_display_name(display_no, True)
+                if display_name:
+                    display_name = display_name.replace("[PRIMARY]",
+                                                        lang.getstr("display.primary"))
+                    self._window.SetTitle(display_name + u" ‒ " +
+                                          lang.getstr("whitepoint.visual_editor"))
         else:
             msg = lang.getstr("whitepoint.visual_editor.display_changed.warning")
             safe_print(msg)
@@ -1771,23 +1826,36 @@ class VisualWhitepointEditor(wx.Frame):
     This is the VisualWhitepointEditor main class implementation.
     """
 
-    def __init__(self, parent, colourData=None, pos=wx.DefaultPosition):
+    def __init__(self, parent, colourData=None, title=wx.EmptyString,
+                 pos=wx.DefaultPosition, patterngenerator=None,
+                 geometry=None, profile=None):
         """
         Default class constructor.
 
         :param `colourData`: a standard :class:`ColourData` (as used in :class:`ColourFrame`);
          to hide the alpha channel control or not.
+        :param `patterngenerator`: a patterngenerator object
+        :param `geometry`: the geometry of the display the profile is assigned to
+        :param `profile`: the profile of the display with the given geometry
         """
+        
+        self.patterngenerator = patterngenerator
+        self.update_patterngenerator_event = threading.Event()
+        
+        style = wx.DEFAULT_FRAME_STYLE
+        if patterngenerator:
+            style &= ~(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX)
 
         wx.Frame.__init__(self, parent, id=wx.ID_ANY,
-                          title=lang.getstr("whitepoint.visual_editor"),
-                          pos=pos, style=wx.DEFAULT_FRAME_STYLE,
+                          title=title or lang.getstr("whitepoint.visual_editor"),
+                          pos=pos, style=style,
                           name="VisualWhitepointEditor")
 
-        self._mgr = AuiManager_LRDocking(self, aui.AUI_MGR_DEFAULT |
-                                         aui.AUI_MGR_LIVE_RESIZE |
-                                         aui.AUI_MGR_SMOOTH_DOCKING)
-        self._mgr.SetArtProvider(AuiDarkDockArt())
+        if not patterngenerator:
+            self._mgr = AuiManager_LRDocking(self, aui.AUI_MGR_DEFAULT |
+                                             aui.AUI_MGR_LIVE_RESIZE |
+                                             aui.AUI_MGR_SMOOTH_DOCKING)
+            self._mgr.SetArtProvider(AuiDarkDockArt())
 
         self.SetIcons(get_icon_bundle([256, 48, 32, 16], appname))
 
@@ -1813,6 +1881,7 @@ class VisualWhitepointEditor(wx.Frame):
         self.mainPanel.BackgroundColour = "#333333"
         self.mainPanel.ForegroundColour = "#999999"
         self.bgPanel = wx_Panel(self, -1)
+        self.bgPanel.Show(not patterngenerator)
 
         self.hsvBitmap = HSVWheel(self.mainPanel,
                                   self.mainPanel.BackgroundColour)
@@ -1847,7 +1916,7 @@ class VisualWhitepointEditor(wx.Frame):
         x, y, scale = (float(v) for v in getcfg("dimensions.measureframe.whitepoint.visual_editor").split(","))
         
         self.area_size_slider = HSlider(self.mainPanel,
-                                        min(scale * 100, 1000), 50, 1000,
+                                        min(scale * 100, 1000), 10, 1000,
                                         self.area_handler)
 
         self.display_size_mm = {}
@@ -1914,63 +1983,73 @@ class VisualWhitepointEditor(wx.Frame):
         # Set up panes
         self.mainPanel.Fit()
         mainPanelSize = (self.mainPanel.Size[0], self.mainPanel.Size[1] + s(10))
-        self._mgr.AddPane(self.mainPanel, aui.AuiPaneInfo().
-                                          Name("mainPanel").
-                                          Fixed().
-                                          Left().
-                                          TopDockable(False).
-                                          BottomDockable(False).
-                                          PaneBorder(False).
-                                          CloseButton(False).
-                                          PinButton(True).
-                                          MinSize(mainPanelSize))
-        self._mgr.AddPane(self.bgPanel, aui.AuiPaneInfo().
-                                        Name("bgPanel").
-                                        CenterPane().
-                                        CloseButton(False).
-                                        PaneBorder(False))
-        self._mgr.Update()
+        if not patterngenerator:
+            self._mgr.AddPane(self.mainPanel, aui.AuiPaneInfo().
+                                              Name("mainPanel").
+                                              Fixed().
+                                              Left().
+                                              TopDockable(False).
+                                              BottomDockable(False).
+                                              PaneBorder(False).
+                                              CloseButton(False).
+                                              PinButton(True).
+                                              MinSize(mainPanelSize))
+            self._mgr.AddPane(self.bgPanel, aui.AuiPaneInfo().
+                                            Name("bgPanel").
+                                            CenterPane().
+                                            CloseButton(False).
+                                            PaneBorder(False))
+            self._mgr.Update()
 
-        self.Bind(aui.EVT_AUI_PANE_CLOSE, self.close_pane_handler)
-        self.Bind(aui.EVT_AUI_PANE_FLOATED, self.float_pane_handler)
-        self.Bind(aui.EVT_AUI_PANE_DOCKED, self.float_pane_handler)
+            self.Bind(aui.EVT_AUI_PANE_CLOSE, self.close_pane_handler)
+            self.Bind(aui.EVT_AUI_PANE_FLOATED, self.float_pane_handler)
+            self.Bind(aui.EVT_AUI_PANE_DOCKED, self.float_pane_handler)
 
         # Account for pane titlebar
         self.Sizer.SetSizeHints(self)
         self.Sizer.Layout()
-        if hasattr(self, "MinClientSize"):
-            # wxPython 2.9+
-            minClientSize = self.MinClientSize
-        else:
-            minClientSize = self.WindowToClientSize(self.MinSize)
-        w, h = self.newColourPanel.Size
-        self.ClientSize = mainPanelSize[0] + w + s(26), max(minClientSize[1], h + s(26))
-        if sys.platform not in ("win32", "darwin"):
-            correction = s(40)
-        else:
-            correction = 0
-        w, h = (int(round(self.default_size)), ) * 2
-        minClientSize = mainPanelSize[0] + max(minClientSize[1], w + s(26)), max(minClientSize[1], h + s(26)) + correction
-        if hasattr(self, "MinClientSize"):
-            # wxPython 2.9+
-            self.MinClientSize = minClientSize
-        else:
-            self.MinSize = self.ClientToWindowSize(minClientSize)
+        if not patterngenerator:
+            if hasattr(self, "MinClientSize"):
+                # wxPython 2.9+
+                minClientSize = self.MinClientSize
+            else:
+                minClientSize = self.WindowToClientSize(self.MinSize)
+            w, h = self.newColourPanel.Size
+            self.ClientSize = mainPanelSize[0] + w + s(26), max(minClientSize[1], h + s(26))
+            if sys.platform not in ("win32", "darwin"):
+                correction = s(40)
+            else:
+                correction = 0
+            w, h = (int(round(self.default_size)), ) * 2
+            minClientSize = mainPanelSize[0] + max(minClientSize[1], w + s(26)), max(minClientSize[1], h + s(26)) + correction
+            if hasattr(self, "MinClientSize"):
+                # wxPython 2.9+
+                self.MinClientSize = minClientSize
+            else:
+                self.MinSize = self.ClientToWindowSize(minClientSize)
 
         x, y = self.Position
         w, h = self.Size
 
-        if (self.newColourPanel.Size[0] > min(self.bgPanel.Size[0],
-                                              self.GetDisplay().ClientArea[2] -
-                                              mainPanelSize[0]) or
-            self.newColourPanel.Size[1] > min(self.bgPanel.Size[1],
-                                              self.GetDisplay().ClientArea[3])):
+        if (not patterngenerator and
+            (self.newColourPanel.Size[0] > min(self.bgPanel.Size[0],
+                                               self.GetDisplay().ClientArea[2] -
+                                               mainPanelSize[0]) or
+             self.newColourPanel.Size[1] > min(self.bgPanel.Size[1],
+                                               self.GetDisplay().ClientArea[3]))):
             w, h = self.GetDisplay().ClientArea[2:]
 
         self.SetSaneGeometry(x, y, w, h)
 
 
-        self._pm = ProfileManager(self)
+        if patterngenerator:
+            self.update_patterngenerator_thread = threading.Thread(
+                target=update_patterngenerator,
+                name="VisualWhitepointEditorPatternGeneratorUpdateThread",
+                args=(self,))
+            self.update_patterngenerator_thread.start()
+
+        self._pm = ProfileManager(self, geometry, profile)
         self.Bind(wx.EVT_MOVE, self.move_handler)
 
         wx.CallAfter(self.InitFrame)
@@ -2075,7 +2154,8 @@ class VisualWhitepointEditor(wx.Frame):
         mainSizer.SetSizeHints(self.mainPanel)
         
         dialogSizer.Add(self.mainPanel, 0, wx.EXPAND)
-        dialogSizer.Add(self.bgPanel, 1, wx.EXPAND, 0)
+        if self.bgPanel.IsShown():
+            dialogSizer.Add(self.bgPanel, 1, wx.EXPAND, 0)
         self.SetAutoLayout(True)
         self.SetSizer(dialogSizer)
         dialogSizer.Fit(self)
@@ -2163,6 +2243,19 @@ class VisualWhitepointEditor(wx.Frame):
         self.brightnessSpin.SetValue(self._colour.v)     
 
         self.SetPanelColours()
+        self.update_patterngenerator()
+
+    def update_patterngenerator(self):
+        if self.patterngenerator:
+            size = min((self.area_size_slider.GetValue() /
+                        float(self.area_size_slider.GetMax() -
+                              self.area_size_slider.GetMin())), 1)
+            x = max(self.area_x_slider.GetValue() /
+                    float(self.area_x_slider.GetMax()) * (1 - size), 0)
+            y = max(self.area_y_slider.GetValue() /
+                    float(self.area_y_slider.GetMax()) * (1 - size), 0)
+            self.patterngenerator_config = x, y, size
+            self.update_patterngenerator_event.set()
         
 
     def SetPanelColours(self):
@@ -2369,6 +2462,7 @@ class VisualWhitepointEditor(wx.Frame):
             wx.CallAfter(self.area_handler)
         else:
             self.bgPanel.Refresh()
+        self.update_patterngenerator()
 
 
     def center_x_handler(self, event):
