@@ -121,10 +121,16 @@ class Tag(object):
 				with open(cachefilename, "rb") as cachefile:
 					body = cachefile.read()
 			if not body.strip():
-				_safe_print("Requesting:", url)
-				response = urllib2.urlopen(url)
-				body = response.read()
-				response.close()
+				for url in (url, url.replace("https://", "http://")):
+					_safe_print("Requesting:", url)
+					try:
+						response = urllib2.urlopen(url)
+					except urllib2.URLError, exception:
+						_safe_print(exception)
+					else:
+						body = response.read()
+						response.close()
+						break
 			if not body.strip():
 				# Fallback to local copy
 				url = get_data_path("x3d-viewer/" + basename)
@@ -152,7 +158,7 @@ class Tag(object):
 					_safe_print("Removing", cachefilename)
 					os.remove(cachefilename)
 		# Get HTML template from cache or online
-		html = get_resource("http://%s/x3d-viewer/release/x3d-viewer.html" %
+		html = get_resource("https://%s/x3d-viewer/release/x3d-viewer.html" %
 							domain.lower(), True)
 		if cache or embed:
 			# Update resources in HTML
@@ -171,7 +177,9 @@ class Tag(object):
 					html = html.replace(restag, resource)
 				else:
 					updated_restag = re.sub(r"(\s+data-fallback-%s=)([\"']).+?\2"
-											% attrname, r"\1\2%s\2" % resource,
+											% attrname,
+											create_replace_function(r"\1\2%s\2",
+																	resource),
 											restag)
 					html = html.replace(restag, updated_restag)
 		# Update title
@@ -365,6 +373,15 @@ def update_vrml(vrml, colorspace):
 			x, y, z = ((x + offsetx) * scale,
 					   (y + offsety) * scale,
 					   Y / 100.0 * maxz)
+		elif colorspace == "ICtCp":
+			I, Ct, Cp = colormath.XYZ2ICtCp(X / 100.0, Y / 100.0, Z / 100.0,
+										  clamp=False)
+			z, x, y = I * 100, Ct * 100, Cp * 100
+		elif colorspace == "IPT":
+			I, P, T = colormath.XYZ2IPT(X / 100.0, Y / 100.0, Z / 100.0)
+			z, x, y = I * 100, P * 100, T * 100
+		elif colorspace == "Lpt":
+			z, x, y = colormath.XYZ2Lpt(X, Y, Z)
 		z -= maxz / 2.0
 		return " ".join(["%.6f" % v for v in (x, y, z)])
 	# Update point lists
@@ -450,6 +467,43 @@ Transform {
 											  offsetx * scale, offsety * scale,
 											  0, maxxy, maxxy, maxz),
 					  vrml)
+	elif colorspace == "ICtCp":
+		# Replace L* a* b* labels with I Ct Cp
+		vrml = re.sub(r'(string\s*\["[+\-]?)L\*?',
+					  r"\1I", vrml)
+		vrml = re.sub(r'(string\s*\["[+\-]?)a\*?',
+					  r"\1Ct", vrml)
+		vrml = re.sub(r'(string\s*\["[+\-]?)b\*?',
+					  r"\1Cp", vrml)
+		# Change axis colors
+		axes = re.findall(r'Shape\s*\{\s*geometry\s*(?:Box|Text)\s*\{\s*(?:size\s+\d+\.0+\s+\d+\.0+\s+\d+\.0+|string\s+\["[^"]*"\]\s*fontStyle\s+FontStyle\s*\{[^}]+\})\s*\}\s*appearance\s+Appearance\s*\{\s*material\s*Material\s*\{[^}]+}\s*\}\s*\}', vrml)
+		for i, axis in enumerate(axes):
+			# Red -> purpleish blue
+			vrml = vrml.replace(axis, re.sub("diffuseColor\s+1\.0+\s+0\.0+\s+0\.0+",
+											 "diffuseColor 0.5 0.0 1.0", axis))
+			# Green -> yellowish green
+			vrml = vrml.replace(axis, re.sub("diffuseColor\s+0\.0+\s+1\.0+\s+0\.0+",
+											 "diffuseColor 0.8 1.0 0.0", axis))
+			# Yellow -> magentaish red
+			vrml = vrml.replace(axis, re.sub("diffuseColor\s+1\.0+\s+1\.0+\s+0\.0+",
+											 "diffuseColor 1.0 0.0 0.25", axis))
+			# Blue -> cyan
+			vrml = vrml.replace(axis, re.sub("diffuseColor\s+0\.0+\s+0\.0+\s+1\.0+",
+											 "diffuseColor 0.0 1.0 1.0", axis))
+	elif colorspace == "IPT":
+		# Replace L* a* b* labels with I P T
+		vrml = re.sub(r'(string\s*\["[+\-]?)L\*?',
+					  r"\1I", vrml)
+		vrml = re.sub(r'(string\s*\["[+\-]?)a\*?',
+					  r"\1P", vrml)
+		vrml = re.sub(r'(string\s*\["[+\-]?)b\*?',
+					  r"\1T", vrml)
+	elif colorspace == "Lpt":
+		# Replace a* b* labels with p* t*
+		vrml = re.sub(r'(string\s*\["[+\-]?)a\*?',
+					  r"\1p", vrml)
+		vrml = re.sub(r'(string\s*\["[+\-]?)b\*?',
+					  r"\1t", vrml)
 	return vrml
 
 
