@@ -3,7 +3,7 @@
 function createpreset() {
 	local root="`dirname \"$0\"`/.."
 	echo "$1"
-	colprof  -ql -aG -C "Created with DisplayCAL and Argyll CMS" -D "DisplayCAL calibration preset: $1" "$root/misc/ti3/$2"
+	colprof  -ql -aG -C "Created with DisplayCAL and ArgyllCMS" -D "DisplayCAL calibration preset: $1" "$root/misc/ti3/$2"
 	mv -i "$root/misc/ti3/$2".ic? "$root/DisplayCAL/presets/$2.icc" && python "$root/util/update_presets.py" "$2"
 	echo ""
 }
@@ -12,9 +12,9 @@ createpreset "Default"      "default"
 createpreset "eeColor"      "video_eeColor"
 createpreset "Laptop"       "laptop"
 createpreset "madVR"        "video_madVR"
+createpreset "madVR ST.2084" "video_madVR_ST2084"
 createpreset "Office & Web" "office_web"
 createpreset "Photo"        "photo"
-createpreset "Prepress"     "prepress"
 createpreset "Prisma"       "video_Prisma"
 createpreset "ReShade"      "video_ReShade"
 createpreset "Resolve"      "video_resolve"
